@@ -18,10 +18,10 @@ init_gvars()
 {
 	domain_check = false;
 	high_prec = false;
-	export_flag = false;
 	partial_flag = true;
 	symb_flag = false;
 	sign_flag = false;
+	in_calc_cmd = false;
 }
 
 /*
@@ -42,6 +42,8 @@ clean_up()
 	for (i = 0; i < n_equations; i++) {
 		if (n_rhs[i] <= 0)
 			n_lhs[i] = 0;
+		if (n_lhs[i] == 1 && lhs[i][0].kind == VARIABLE && lhs[i][0].token.variable == SPECIAL)
+			n_lhs[i] = 0;
 	}
 }
 
@@ -102,16 +104,7 @@ int
 always_positive(power)
 double		power;
 {
-	double	d, d1;
-
-#if	false
-	power = fabs(power);
-	d1 = power * epsilon;
-	d = fmod(power, 2.0);
-	return(d <= d1 || d > (2.0 - d1));
-#else
 	return(fmod(power, 2.0) == 0.0);
-#endif
 }
 
 /*
@@ -151,13 +144,9 @@ int	want, have;
 	int		rv;
 
 	if (n_lhs[want] > 0) {
-		rv = solve_sub(&lhs[want][0], n_lhs[want],
-		    &lhs[have][0], &n_lhs[have],
-		    &rhs[have][0], &n_rhs[have]);
+		rv = solve_sub(lhs[want], n_lhs[want], lhs[have], &n_lhs[have], rhs[have], &n_rhs[have]);
 	} else {
-		rv = solve_sub(&rhs[want][0], n_rhs[want],
-		    &rhs[have][0], &n_rhs[have],
-		    &lhs[have][0], &n_lhs[have]);
+		rv = solve_sub(rhs[want], n_rhs[want], rhs[have], &n_rhs[have], lhs[have], &n_lhs[have]);
 	}
 	n_lhs[want] = 0;
 	if (!rv)
@@ -207,9 +196,9 @@ int		*rightnp;	/* size of RHS */
 	uf_flag = false;
 	inc_count = 0;
 	n_tlhs = *leftnp;
-	blt(&tlhs[0], leftp, n_tlhs * sizeof(*leftp));
+	blt(tlhs, leftp, n_tlhs * sizeof(*leftp));
 	n_trhs = *rightnp;
-	blt(&trhs[0], rightp, n_trhs * sizeof(*rightp));
+	blt(trhs, rightp, n_trhs * sizeof(*rightp));
 	if (wantn != 1) {
 #if	!SILENT
 		printf(_("This program will only solve for a single variable or for zero.\n"));
@@ -226,14 +215,14 @@ int		*rightnp;	/* size of RHS */
 		v = wantp->token.variable;
 		found = false;
 		ep = &trhs[n_trhs];
-		for (p1 = &trhs[0]; p1 < ep; p1 += 2) {
+		for (p1 = trhs; p1 < ep; p1 += 2) {
 			if (p1->kind == VARIABLE && p1->token.variable == v) {
 				found = true;
 				break;
 			}
 		}
 		ep = &tlhs[n_tlhs];
-		for (p1 = &tlhs[0]; p1 < ep; p1 += 2) {
+		for (p1 = tlhs; p1 < ep; p1 += 2) {
 			if (p1->kind == VARIABLE && p1->token.variable == v) {
 				found = true;
 				break;
@@ -256,36 +245,39 @@ int		*rightnp;	/* size of RHS */
 		}
 		zsolve = true;
 	}
-	uf_power(&tlhs[0], &n_tlhs);
-	uf_power(&trhs[0], &n_trhs);
+#if	true
+	uf_power(tlhs, &n_tlhs);
+	uf_power(trhs, &n_trhs);
+#endif
 simp_again:
 	list_tdebug(2);
-	simps_side(&tlhs[0], &n_tlhs, zsolve);
+	simps_side(tlhs, &n_tlhs, zsolve);
 	if (uf_flag) {
-		simp_loop(&trhs[0], &n_trhs);
-		uf_simp(&trhs[0], &n_trhs);
-		factorv(&trhs[0], &n_trhs, v);
+		simp_loop(trhs, &n_trhs);
+		uf_simp(trhs, &n_trhs);
+		factorv(trhs, &n_trhs, v);
 	} else {
-		simps_side(&trhs[0], &n_trhs, zsolve);
+		simps_side(trhs, &n_trhs, zsolve);
 	}
 	list_tdebug(1);
 no_simp:
+	/* Move from the RHS to the LHS. */
 	op = 0;
 	ep = &trhs[n_trhs];
 	if (zsolve) {
-		for (b1 = p1 = &trhs[0]; p1 < ep; p1++) {
+		for (b1 = p1 = trhs; p1 < ep; p1++) {
 			if (p1->level == 1 && p1->kind == OPERATOR) {
 				op = p1->token.operatr;
 				b1 = p1 + 1;
 				if (op == DIVIDE) {
-					if (!g_of_f(op, b1, &trhs[0], &n_trhs, &tlhs[0], &n_tlhs))
+					if (!g_of_f(op, b1, trhs, &n_trhs, tlhs, &n_tlhs))
 						return false;
 					goto simp_again;
 				}
 			}
 		}
 	} else {
-		for (b1 = p1 = &trhs[0]; p1 < ep; p1++) {
+		for (b1 = p1 = trhs; p1 < ep; p1++) {
 			if (p1->kind == VARIABLE && v == p1->token.variable) {
 				if (op == 0) {
 					for (p1++;; p1++) {
@@ -311,13 +303,13 @@ no_simp:
 						}
 					}
 				}
-				if (op == TIMES /* || op == DIVIDE */) {
-					b1 = &trhs[0];
+				if (op == TIMES || op == DIVIDE || op == POWER) {
+					b1 = trhs;
 					op = PLUS;
 					for (p1 = b1; p1 < ep; p1++)
 						p1->level++;
 				}
-				if (!g_of_f(op, b1, &trhs[0], &n_trhs, &tlhs[0], &n_tlhs))
+				if (!g_of_f(op, b1, trhs, &n_trhs, tlhs, &n_tlhs))
 					return false;
 				goto simp_again;
 			} else if (p1->level == 1 && p1->kind == OPERATOR) {
@@ -327,27 +319,29 @@ no_simp:
 		}
 	}
 	if (uf_flag) {
-		simps_side(&trhs[0], &n_trhs, zsolve);
+		simps_side(trhs, &n_trhs, zsolve);
 	}
 left_again:
 	worked = 1;
 	uf_flag = false;
 see_work:
-	if (se_compare(wantp, wantn, &tlhs[0], n_tlhs, &diff_sign)) {
+	/* See if we have solved the equation. */
+	if (se_compare(wantp, wantn, tlhs, n_tlhs, &diff_sign)) {
 		if (!diff_sign) {
 			if (!zsolve) {
 				ep = &trhs[n_trhs];
-				for (p1 = &trhs[0]; p1 < ep; p1 += 2) {
-					if (p1->kind == VARIABLE && p1->token.variable == v)
-						goto fool2;
+				for (p1 = trhs; p1 < ep; p1 += 2) {
+					if (p1->kind == VARIABLE && p1->token.variable == v) {
+						return false;
+					}
 				}
 			} else {
 zero_simp:
 				list_tdebug(4);
-				uf_power(&trhs[0], &n_trhs);
+				uf_power(trhs, &n_trhs);
 				do {
-					simp_ssub(&trhs[0], &n_trhs, 0L, 0.0, false, 2, true, 0);
-				} while (uf_power(&trhs[0], &n_trhs));
+					simp_ssub(trhs, &n_trhs, 0L, 0.0, false, 2, true, 0);
+				} while (uf_power(trhs, &n_trhs));
 				list_tdebug(3);
 				ep = &trhs[n_trhs];
 				op = 0;
@@ -364,7 +358,7 @@ zero_simp:
 				}
 				switch (op) {
 				case TIMES:
-					for (p1 = &trhs[0]; p1 < ep; p1 += 2) {
+					for (p1 = trhs; p1 < ep; p1 += 2) {
 						if (p1->kind == CONSTANT
 						    && p1->level == 1
 						    && p1->token.constant != 1.0) {
@@ -395,6 +389,7 @@ zero_simp:
 					break;
 				}
 			}
+#if	!SILENT
 			if (debug_level > 0) {
 				if (zsolve) {
 					printf(_("Solve for zero completed:\n"));
@@ -402,21 +397,22 @@ zero_simp:
 					printf(_("Solve completed:\n"));
 				}
 			}
+#endif
 			list_tdebug(1);
-			blt(leftp, &tlhs[0], n_tlhs * sizeof(*leftp));
+			blt(leftp, tlhs, n_tlhs * sizeof(*leftp));
 			*leftnp = n_tlhs;
-			blt(rightp, &trhs[0], n_trhs * sizeof(*rightp));
+			blt(rightp, trhs, n_trhs * sizeof(*rightp));
 			*rightnp = n_trhs;
 			return true;
 		}
 	}
-fool2:
+	/* Move what we don't want in the LHS to the RHS. */
 	found_count = 0;
 	need_flip = 0;
 	found = 0;
 	op = 0;
 	ep = &tlhs[n_tlhs];
-	for (b1 = p1 = &tlhs[0];; p1++) {
+	for (b1 = p1 = tlhs;; p1++) {
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
 			if (!found) {
 				if ((p1 < ep || found_count || zsolve || n_tlhs > 1 || tlhs[0].kind != CONSTANT)
@@ -459,22 +455,22 @@ fool2:
 							}
 						} else {
 							if (op != DIVIDE) {
-								b1 = &tlhs[0];
+								b1 = tlhs;
 								op = PLUS;
 								for (p1 = b1; p1 < ep; p1++)
 									p1->level++;
 							}
 						}
 					}
-					if (!g_of_f(op, b1, &tlhs[0], &n_tlhs, &trhs[0], &n_trhs))
+					if (!g_of_f(op, b1, tlhs, &n_tlhs, trhs, &n_trhs))
 						return false;
 					list_tdebug(2);
 					if (uf_flag) {
-						simp_loop(&tlhs[0], &n_tlhs);
+						simp_loop(tlhs, &n_tlhs);
 					} else {
-						simps_side(&tlhs[0], &n_tlhs, zsolve);
+						simps_side(tlhs, &n_tlhs, zsolve);
 					}
-					simps_side(&trhs[0], &n_trhs, zsolve);
+					simps_side(trhs, &n_trhs, zsolve);
 					list_tdebug(1);
 					goto see_work;
 				}
@@ -482,19 +478,21 @@ fool2:
 				need_flip += found;
 			}
 			if (p1 >= ep) {
-				if (found_count == 0) {
-					for (i = 0; i < n_trhs; i++) {
+				if (found_count == 0) { /* if solve variable no longer in LHS */
+					for (i = 0; i < n_trhs; i += 2) {
 						if (trhs[i].kind == VARIABLE
-						    && trhs[i].token.variable == v)
+						    && trhs[i].token.variable == v) {
+							/* solve variable in RHS */
 							return false;
+						}
 					}
-					subst_constants(&tlhs[0], &n_tlhs);
-					subst_constants(&trhs[0], &n_trhs);
-					simps_side(&tlhs[0], &n_tlhs, zsolve);
-					simps_side(&trhs[0], &n_trhs, zsolve);
-					uf_simp(&tlhs[0], &n_tlhs);
-					uf_simp(&trhs[0], &n_trhs);
-					if (se_compare(&tlhs[0], n_tlhs, &trhs[0], n_trhs, &diff_sign) && !diff_sign) {
+					subst_constants(tlhs, &n_tlhs);
+					subst_constants(trhs, &n_trhs);
+					simps_side(tlhs, &n_tlhs, zsolve);
+					simps_side(trhs, &n_trhs, zsolve);
+					uf_simp(tlhs, &n_tlhs);
+					uf_simp(trhs, &n_trhs);
+					if (se_compare(tlhs, n_tlhs, trhs, n_trhs, &diff_sign) && !diff_sign) {
 #if	!SILENT
 						printf(_("\nThis equation is an identity; that is,\n"));
 						printf(_("the left equation side is identical to the right equation side.\n"));
@@ -519,53 +517,51 @@ fool2:
 					if (found) {
 #if	!SILENT
 						printf(_("This equation is independent of ("));
-						list_proc(&wantp[0], wantn);
-						printf(_(").\n"));
+						list_proc(wantp, wantn, false);
+						printf(").\n");
 #endif
 						return false;
 					}
-no_val:
 #if	!SILENT
 					printf(_("There are no possible values for ("));
-					list_proc(&wantp[0], wantn);
-					printf(_(").\n"));
+					list_proc(wantp, wantn, false);
+					printf(").\n");
 #endif
 					return false;
 				}
 				zflag = (n_trhs == 1 && trhs[0].kind == CONSTANT
 				    && trhs[0].token.constant == 0.0);
 				if (need_flip >= found_count) {
-					if (zflag)
-						goto no_val;
-					if (!flip(&tlhs[0], &n_tlhs, &trhs[0], &n_trhs))
+					if (zflag) {
+						/* overwrite -0.0 */
+						trhs[0].token.constant = 0.0;
+					}
+					if (!flip(tlhs, &n_tlhs, trhs, &n_trhs))
 						return false;
 					list_tdebug(2);
-					simps_side(&tlhs[0], &n_tlhs, zsolve);
-					simps_side(&trhs[0], &n_trhs, zsolve);
+					simps_side(tlhs, &n_tlhs, zsolve);
+					simps_side(trhs, &n_trhs, zsolve);
 					list_tdebug(1);
 					goto left_again;
 				}
 				if (worked && !uf_flag) {
 					worked--;
-#if	!SILENT
-					printf(_("Unfactoring...\n"));
-#endif
+					debug_string(1, "Unfactoring...");
 					partial_flag = false;
-					uf_simp(&tlhs[0], &n_tlhs);
+					uf_simp(tlhs, &n_tlhs);
 					partial_flag = true;
-					factorv(&tlhs[0], &n_tlhs, v);
+					factorv(tlhs, &n_tlhs, v);
 					list_tdebug(1);
 					uf_flag = true;
 					goto see_work;
 				}
 				if (uf_flag) {
-					simps_side(&tlhs[0], &n_tlhs, zsolve);
+					simps_side(tlhs, &n_tlhs, zsolve);
 					uf_flag = false;
 					goto see_work;
 				}
-split_solve:
 				op = 0;
-				b1 = &tlhs[0];
+				b1 = tlhs;
 				for (i = 1; i < n_tlhs; i += 2) {
 					if (tlhs[i].level == 1) {
 						op_kind = tlhs[i].token.operatr;
@@ -577,7 +573,7 @@ split_solve:
 							op = op_kind;
 							break;
 						}
-						if (zflag && inc_count) {
+						if (zflag) {
 							if (op_kind == DIVIDE
 							    || (tlhs[i+1].level == 1 && tlhs[i+1].kind != CONSTANT)) {
 								op = op_kind;
@@ -601,30 +597,26 @@ split_solve:
 						}
 					}
 				}
-				if ((zflag && inc_count && op == TIMES
+				if ((zflag && zero_solved && op == TIMES
 				    && b1->level == 1 && b1->kind != CONSTANT)
 				    || op == DIVIDE) {
 					if (op == TIMES) {
-						if (inc_count)
-							inc_count--;
 						qtries = 0;	/* might be quadratic after removing solution */
 #if	!SILENT
 						printf(_("Removing possible solution: \""));
-						k = b1 - &tlhs[0];
+						k = b1 - tlhs;
 						for (j = k + 1; j < n_tlhs; j += 2)
 							if (tlhs[j].level <= 1)
 								break;
-						list_proc(b1, j - k);
-						printf(_(" = 0\".\n"));
+						list_proc(b1, j - k, false);
+						printf(" = 0\".\n");
 #endif
 					} else {
-#if	!SILENT
-						printf(_("Juggling...\n"));
-#endif
+						debug_string(1, "Juggling...");
+						uf_flag = true;
 					}
-					if (!g_of_f(op, b1, &tlhs[0], &n_tlhs, &trhs[0], &n_trhs))
+					if (!g_of_f(op, b1, tlhs, &n_tlhs, trhs, &n_trhs))
 						return false;
-					uf_flag = true;
 					goto simp_again;
 				}
 				b1 = NULL;
@@ -632,10 +624,9 @@ split_solve:
 					if (tlhs[i].token.operatr == POWER
 					    && tlhs[i+1].level == tlhs[i].level
 					    && tlhs[i+1].kind == CONSTANT
-					    && tlhs[i+1].token.constant < 1.0
-					    && tlhs[i+1].token.constant > -1.0) {
+					    && fabs(tlhs[i+1].token.constant) < 1.0) {
 						if (!f_to_fraction(tlhs[i+1].token.constant, &numerator, &denominator)
-						    || numerator != 1.0) {
+						    || fabs(numerator) != 1.0 || denominator < 2.0) {
 							continue;
 						}
 						for (j = i - 1; (j >= 0) && tlhs[j].level >= tlhs[i].level; j--) {
@@ -666,18 +657,13 @@ split_solve:
 					goto simp_again;
 				}
 				if (qtries) {
-					if (inc_count || !zflag)
-						return false;
-					inc_count++;
-					goto split_solve;
+					return false;
 				}
-#if	!SILENT
-				printf(_("Solving for zero...\n"));
-#endif
+				debug_string(1, "Solving for zero...");
 				*leftnp = n_tlhs;
-				blt(leftp, &tlhs[0], n_tlhs * sizeof(*leftp));
+				blt(leftp, tlhs, n_tlhs * sizeof(*leftp));
 				*rightnp = n_trhs;
-				blt(rightp, &trhs[0], n_trhs * sizeof(*rightp));
+				blt(rightp, trhs, n_trhs * sizeof(*rightp));
 				want0.level = 1;
 				want0.kind = CONSTANT;
 				want0.token.constant = 0.0;
@@ -727,10 +713,10 @@ long	v;
 	int		j, k;
 
 	partial_flag = false;
-	ufactor(&tlhs[0], &n_tlhs);
+	ufactor(tlhs, &n_tlhs);
 	partial_flag = true;
-	simp_ssub(&tlhs[0], &n_tlhs, v, d, true, false, false, 1);
-	simp_ssub(&tlhs[0], &n_tlhs, 0L, 0.0, true, false, true, 1);
+	simp_ssub(tlhs, &n_tlhs, v, d, true, false, false, 1);
+	simp_ssub(tlhs, &n_tlhs, 0L, 0.0, true, false, true, 1);
 lonely:
 	ep = &tlhs[n_tlhs];
 	len1 = 0;
@@ -757,7 +743,7 @@ lonely:
 				    && b1->token.variable == v) {
 					flag = true;
 				}
-				if (b1 == &tlhs[0])
+				if (b1 == tlhs)
 					break;
 			}
 			if (flag) {
@@ -785,7 +771,7 @@ lonely:
 				    && b1->token.variable == v) {
 					flag = true;
 				}
-				if (b1 == &tlhs[0])
+				if (b1 == tlhs)
 					break;
 			}
 			if (flag) {
@@ -804,17 +790,17 @@ lonely:
 	op = p1->token.operatr;
 	if (op == POWER && b1->level == 1
 	    && b1->kind == CONSTANT && b1->token.constant == d) {
-		return(g_of_f(POWER, b1, &tlhs[0], &n_tlhs, &trhs[0], &n_trhs));
+		return(g_of_f(POWER, b1, tlhs, &n_tlhs, trhs, &n_trhs));
 	}
 	if (!foundp) {
-		b1 = &tlhs[0];
+		b1 = tlhs;
 		if (p1 - b1 == 1 && p1->token.operatr == DIVIDE
 		    && b1->kind == CONSTANT && b1->token.constant == 1.0) {
-			if (!flip(&tlhs[0], &n_tlhs, &trhs[0], &n_trhs))
+			if (!flip(tlhs, &n_tlhs, trhs, &n_trhs))
 				return false;
 			list_tdebug(2);
-			simp_loop(&tlhs[0], &n_tlhs);
-			simp_loop(&trhs[0], &n_trhs);
+			simp_loop(tlhs, &n_tlhs);
+			simp_loop(trhs, &n_trhs);
 			list_tdebug(1);
 			goto lonely;
 		}
@@ -832,9 +818,10 @@ lonely:
 			break;
 		}
 	}
+#if	!SILENT
 	if (op == TIMES && zflag) {
 		foundp = false;
-		k = b1 - &tlhs[0];
+		k = b1 - tlhs;
 		for (j = k + 1;; j += 2) {
 			if (tlhs[j-1].kind == VARIABLE
 			    && tlhs[j-1].token.variable == v)
@@ -842,19 +829,18 @@ lonely:
 			if (j >= n_tlhs || tlhs[j].level <= 1)
 				break;
 		}
-#if	!SILENT
 		if (foundp) {
 			printf(_("Removing possible solution: \""));
-			list_proc(b1, j - k);
-			printf(_(" = 0\".\n"));
+			list_proc(b1, j - k, false);
+			printf(" = 0\".\n");
 		}
-#endif
 	}
-	if (!g_of_f(op, b1, &tlhs[0], &n_tlhs, &trhs[0], &n_trhs))
+#endif
+	if (!g_of_f(op, b1, tlhs, &n_tlhs, trhs, &n_trhs))
 		return false;
 	list_tdebug(2);
-	simp_loop(&tlhs[0], &n_tlhs);
-	simp_loop(&trhs[0], &n_trhs);
+	simp_loop(tlhs, &n_tlhs);
+	simp_loop(trhs, &n_trhs);
 	list_tdebug(1);
 	goto lonely;
 }
@@ -886,15 +872,15 @@ long	v;
 	int		aloc;
 	double		high_power;
 
-	uf_simp(&trhs[0], &n_trhs);
-	factorv(&trhs[0], &n_trhs, v);
+	uf_simp(trhs, &n_trhs);
+	factorv(trhs, &n_trhs, v);
 	list_tdebug(1);
 
 	found = false;
 	op = 0;
 	high_power = 0.0;
 	ep = &trhs[n_trhs];
-	for (x1tp = p1 = &trhs[0];; p1++) {
+	for (x1tp = p1 = trhs;; p1++) {
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
 			if (p1 < ep) {
 				switch (p1->token.operatr) {
@@ -970,15 +956,14 @@ long	v;
 			}
 		}
 	}
-	if (a1p > &trhs[0] && (a1p - 1)->token.operatr == MINUS)
+	if (a1p > trhs && (a1p - 1)->token.operatr == MINUS)
 		opx1 = MINUS;
 	else
 		opx1 = PLUS;
 #if	!SILENT
 	if (high_power > 1.0) {
-		printf(_("The highest power of ("));
-		list_var(v, true, false);
-		printf(_(") in equation is %g\n"), high_power);
+		list_var(v, false, false);
+		printf(_("The highest power of (%s) in equation is %g\n"), var_str, high_power);
 	}
 #endif
 	if (high_power == 2.0) {
@@ -995,7 +980,7 @@ long	v;
 	}
 	opx2 = 0;
 	op = 0;
-	for (x2p = p1 = &trhs[0];; p1++) {
+	for (x2p = p1 = trhs;; p1++) {
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
 			if (se_compare(x1_storage, nx1, x2p, p1 - x2p, &diff_sign)) {
 				b1p = x2p;
@@ -1057,7 +1042,7 @@ big_bbreak:
 	default:
 		return false;
 	}
-	blt(&scratch[0], b1p, (int) ((char *) x2p - (char *) b1p));
+	blt(scratch, b1p, (int) ((char *) x2p - (char *) b1p));
 	len = x2p - b1p;
 	scratch[len].level = 7;
 	scratch[len].kind = CONSTANT;
@@ -1069,7 +1054,7 @@ big_bbreak:
 	blt(&scratch[len], b2p, (int) ((char *) b2ep - (char *) b2p));
 	len += (b2ep - b2p);
 	blen = len;
-	j = min_level(&scratch[0], len);
+	j = min_level(scratch, len);
 	j = 7 - j;
 	for (i = 0; i < len; i++)
 		scratch[i].level += j;
@@ -1119,11 +1104,11 @@ big_bbreak:
 	scratch[len].kind = CONSTANT;
 	scratch[len].token.constant = 0.0;
 	len++;
-	for (p2 = p1 = &trhs[0];; p1++) {
+	for (p2 = p1 = trhs;; p1++) {
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
 			if ((p2 <= x1p && p1 > x1p) || (p2 <= x2p && p1 > x2p))
 				goto skip_this;
-			if (p2 == &trhs[0]) {
+			if (p2 == trhs) {
 				scratch[len].level = 1;
 				scratch[len].kind = OPERATOR;
 				scratch[len].token.operatr = PLUS;
@@ -1163,7 +1148,7 @@ skip_this:
 	if (len + blen + 3 + alen > n_tokens) {
 		error_huge();
 	}
-	blt(&scratch[len], &scratch[0], blen * sizeof(token_type));
+	blt(&scratch[len], scratch, blen * sizeof(token_type));
 	len += blen;
 	scratch[len].level = 1;
 	scratch[len].kind = OPERATOR;
@@ -1183,18 +1168,16 @@ skip_this:
 		if (scratch[i].kind == VARIABLE && scratch[i].token.variable == v)
 			return false;
 	}
-	if (debug_level > 0) {
-		printf(_("Plugging equation into the quadratic formula:\n"));
-	}
-	blt(&tlhs[0], x1_storage, nx1 * sizeof(token_type));
+	debug_string(1, "Plugging equation into the quadratic formula:");
+	blt(tlhs, x1_storage, nx1 * sizeof(token_type));
 	n_tlhs = nx1;
-	simp_loop(&tlhs[0], &n_tlhs);
-	blt(&trhs[0], &scratch[0], len * sizeof(token_type));
+	simp_loop(tlhs, &n_tlhs);
+	blt(trhs, scratch, len * sizeof(token_type));
 	n_trhs = len;
-	simp_loop(&trhs[0], &n_trhs);
+	simp_loop(trhs, &n_trhs);
 	list_tdebug(2);
-	uf_simp(&trhs[0], &n_trhs);
-	simps_side(&trhs[0], &n_trhs, false);
+	uf_simp(trhs, &n_trhs);
+	simps_side(trhs, &n_trhs, false);
 	list_tdebug(1);
 #if	!SILENT
 	if (high_power == 2.0) {
@@ -1231,24 +1214,22 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	double		numerator, denominator;
 	static int	prev_n1, prev_n2;
 
-	if (debug_level >= 5) {
-		printf(_("Entering g_of_f().\n"));
-	}
 	oldn = *side1np;
 	ep = &side1p[oldn];
+#if	!SILENT
 	if (operandp < side1p || operandp >= ep
 	    || (operandp > side1p && ((operandp - 1)->kind != OPERATOR
 	    || (operandp - 1)->level != 1))) {
 		printf(_("Bug in call to g_of_f()!\n"));
 		return false;
 	}
+#endif
 	if (*side1np == prev_n1 && *side2np == prev_n2) {
 		if (++repeat_count >= 5) {
 #if	!SILENT
-			printf(_("Infinite loop aborted in solve routine!\n"));
+			printf(_("Infinite loop aborted in solve routine.\n"));
 #endif
-			repeat_count = 0;
-			longjmp(jmp_save, 2);
+			return false;
 		}
 	} else {
 		prev_n1 = *side1np;
@@ -1268,9 +1249,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		if (i >= 0 && side1p[i].level == 1 && side1p[i].token.operatr == op) {
 			i = *side2np - 2;
 			if (i >= 0 && side2p[i].level == 1 && side2p[i].token.operatr == op) {
-				if (debug_level > 0) {
-					printf(_("Removing factorial from both sides of the equation:\n"));
-				}
+				debug_string(1, "Removing factorial from both sides of the equation:");
 				*side1np -= 2;
 				*side2np -= 2;
 				return true;
@@ -1282,16 +1261,48 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	for (p1 = operandp + 1; p1 < ep; p1 += 2) {
 		if (p1->level == 1) {
 			switch (p1->token.operatr) {
-			case POWER:
 			case FACTORIAL:
-			case MODULUS:
 				op = PLUS;
 				continue;
+			case MODULUS:
+				if (op != MODULUS) {
+					printf(_("Something is wrong with the solve routine!\n"));
+				} else {
+					operandp = p1 + 1;
+				}
+				continue;
 			}
 			break;
 		}
 	}
 	operandn = p1 - operandp;
+	if (op == MODULUS) {
+		if (*side2np == 1 && side2p->kind == CONSTANT) {
+			if (side2p->token.constant < 0.0
+			    || (operandn == 1 && operandp->kind == CONSTANT
+			    && side2p->token.constant >= fabs(operandp->token.constant))) {
+#if	!SILENT
+				printf(_("There are no solutions.\n"));
+#endif
+				return false;
+			}
+		}
+	}
+	if (op == POWER && operandp == side1p) {
+		subst_constants(side1p, side1np);
+		subst_constants(side2p, side2np);
+		if (operandn != 1 || operandp->kind != CONSTANT)
+			return false;
+		if (*side2np != 1 || side2p->kind != CONSTANT)
+			return false;
+		debug_string(1, "Taking logarithm of both sides:");
+		side2p->token.constant = log(side2p->token.constant) / log(operandp->token.constant);
+		check_err();
+		blt(side1p, p1 + 1, (*side1np - (operandn + 1)) * sizeof(token_type));
+		*side1np -= operandn + 1;
+		return true;
+	}
+#if	!SILENT
 	if (debug_level > 0) {
 		switch (op) {
 		case PLUS:
@@ -1313,10 +1324,10 @@ int		*side2np;	/* pointer to the length of "side2p" */
 			printf(_("Applying inverse modulus of"));
 			break;
 		}
-		printf(_(" \""));
+		printf(" \"");
 		if (op == POWER)
-			printf(_("1/("));
-		list_proc(operandp, operandn);
+			printf("1/(");
+		list_proc(operandp, operandn, false);
 		switch (op) {
 		case PLUS:
 			printf(_("\" from both sides of the equation:\n"));
@@ -1326,12 +1337,13 @@ int		*side2np;	/* pointer to the length of "side2p" */
 			printf(_("\" to both sides of the equation:\n"));
 			break;
 		case POWER:
-			printf(_(")"));
+			printf(")");
 		default:
-			printf(_("\":\n"));
+			printf("\":\n");
 			break;
 		}
 	}
+#endif
 	if (*side1np + operandn + 3 > n_tokens
 	    || *side2np + operandn + 5 > n_tokens) {
 		error_huge();
@@ -1512,10 +1524,6 @@ int		n;
 	token_type	*p1, *ep;
 
 	if (n <= 1) {
-		if (n <= 0) {
-			printf(_("Programming error in call to min_level()!\n"));
-			longjmp(jmp_save, 2);
-		}
 		return equation[0].level;
 	}
 	min1 = equation[1].level;
@@ -1527,6 +1535,11 @@ int		n;
 	return min1;
 }
 
+/*
+ * This is called when the max expression size has been exceeded.
+ *
+ * There is no return.
+ */
 error_huge()
 {
 	longjmp(jmp_save, 14);
