@@ -44,6 +44,7 @@ int	product_flag;
 	double	d1, d2;
 	int	result_en;
 	int	n;
+	int	count_down;
 
 	v = 0L;
 	if (notdefined(cur_equation)) {
@@ -92,7 +93,7 @@ int	product_flag;
 		list_var(v, false, false);
 		strcpy(prompt_str, var_str);
 		strcat(prompt_str, " = ");
-		if ((cp1 = getstring((char *) &scratch[0], n_tokens * sizeof(token_type))) == NULL)
+		if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
 			return false;
 	}
 	d1 = strtod(cp1, &cp);
@@ -105,7 +106,7 @@ int	product_flag;
 		cp1 = cp;
 	} else {
 		strcpy(prompt_str, _("To: "));
-		if ((cp1 = getstring((char *) &scratch[0], n_tokens * sizeof(token_type))) == NULL)
+		if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
 			return false;
 	}
 	d2 = strtod(cp1, &cp);
@@ -113,10 +114,9 @@ int	product_flag;
 		printf(_("Error: not an integer.\n"));
 		return false;
 	}
-	if (d2 <= d1) {
-		printf(_("Range error.\n"));
+	if (extra_garbage(cp))
 		return false;
-	}
+	count_down = (d2 < d1);
 	rhs[result_en][0].kind = CONSTANT;
 	rhs[result_en][0].level = 1;
 	if (product_flag) {
@@ -125,7 +125,7 @@ int	product_flag;
 		rhs[result_en][0].token.constant = 0.0;
 	}
 	n = 1;
-	for (; d1 <= d2; d1 += 1.0) {
+	for (; count_down ? (d1 >= d2) : (d1 <= d2); count_down ? (d1 -= 1.0) : (d1 += 1.0)) {
 		if (n + 1 + n_rhs[cur_equation] > n_tokens) {
 			error_huge();
 		}
@@ -156,7 +156,7 @@ int	product_flag;
 		calc_simp(rhs[result_en], &n);
 	}
 	if (n == 1 && rhs[result_en][0].kind == CONSTANT) {
-		list_proc(rhs[result_en], n);
+		list_proc(rhs[result_en], n, false);
 		fprintf(gfp, "\n");
 	} else {
 		n_rhs[result_en] = n;
@@ -392,57 +392,6 @@ char	*cp;
 	return rv;
 }
 
-int
-ngcd(d1, d2)
-double	d1, d2;
-{
-	double	d3, d4;
-	double	d5, d6;
-
-	if (fabs(d1) >= 1e14 || fabs(d2) >= 1e14) {
-		printf(_("Constant too large!\n"));
-		return false;
-	}
-	d3 = gcd(d1, d2);
-	if (d3 == 0.0) {
-		printf(_("No GCD found.\n"));
-		return false;
-	}
-	if (fabs(d1 / d3) < (1.0 - epsilon)
-	    || fabs(d2 / d3) < (1.0 - epsilon)) {
-		printf(_("The computed GCD is too large!\n"));
-		return false;
-	}
-	modf(fabs(d1 / d3) + 0.5, &d4);
-	d4 = fabs(d1 / d4);
-	if (fabs(modf(fabs(d1 / d4), &d5) - 0.5) < (0.5 - epsilon)
-	    || fabs(modf(fabs(d2 / d4), &d6) - 0.5) < (0.5 - epsilon)) {
-		printf(_("The computed GCD is too inaccurate!\n"));
-		return false;
-	}
-	modf(fabs(d1 / d4) + 0.5, &d5);
-	modf(fabs(d2 / d4) + 0.5, &d6);
-	if (gcd(d5, d6) != 1.0) {
-		printf(_("The computed GCD is too small!\n"));
-		return false;
-	}
-	printf(_("Greatest Common Divisor (GCD) = %.14lg\n"), d4);
-	printf(_("Least Common Multiple (LCM) = %.14lg\n"), fabs((d1 * d2) / d4));
-	return true;
-}
-
-int
-isno(cp)
-char	*cp;
-{
-	if (strncasecmp(cp, "no", 2) == 0
-	    || strncasecmp(cp, "off", 3) == 0
-	    || strncasecmp(cp, "false", 5) == 0) {
-		return true;
-	}
-	return false;
-}
-
 /*
  * The set command.
  */
@@ -450,18 +399,17 @@ int
 set_cmd(cp)
 char	*cp;
 {
-	int	negate;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
 		printf(_("Options are set as follows:\n\n"));
-
-		printf("debug = %d\n", debug_level);
-
+#if	!SILENT
+		printf("debug_level = %d\n", debug_level);
+#endif
 		if (!case_sensitive_flag) {
 			printf("no ");
 		}
-		printf("case\n");
+		printf("case_sensitive\n");
 
 		if (!color_flag) {
 			printf("no ");
@@ -480,6 +428,16 @@ char	*cp;
 		}
 		printf("preserve_roots\n");
 
+		if (!true_modulus) {
+			printf("no ");
+		}
+		printf("true_modulus\n");
+
+		if (!finance_option) {
+			printf("no ");
+		}
+		printf("finance\n");
+
 #if	!SECURE
 		if (getcwd(buf, sizeof(buf))) {
 			printf("directory = %s\n", buf);
@@ -487,21 +445,51 @@ char	*cp;
 #endif
 		return true;
 	}
+	return set_options(cp);
+}
+
+int
+isno(cp)
+char	*cp;
+{
+	if (strcmp_tospace(cp, "no") == 0
+	    || strcmp_tospace(cp, "off") == 0
+	    || strcmp_tospace(cp, "false") == 0) {
+		return true;
+	}
+	return false;
+}
+
+/*
+ * Handle parsing of options for "set" command.
+ *
+ * Return false if error.
+ */
+int
+set_options(cp)
+char	*cp;
+{
+	int	negate;
+
+	cp = skip_space(cp);
+	if (*cp == '\0') {
+		return true;
+	}
 	negate = isno(cp);
 	if (negate) {
 		cp = skip_param(cp);
 	}
 #if	!SECURE
-	if (strncasecmp(cp, "directory", 3) == 0) {
+	if (strncasecmp(cp, "dir", 3) == 0) {
 		cp = skip_param(cp);
 		if (chdir(cp)) {
 			printf(_("Error changing directory.\n"));
 			return false;
 		}
-		printf(_("Current directory changed to \"%s\".\n"), cp);
 		return true;
 	}
 #endif
+#if	!SILENT
 	if (strncasecmp(cp, "debug", 5) == 0) {
 		cp = skip_param(cp);
 		negate = (negate || isno(cp));
@@ -514,12 +502,9 @@ char	*cp;
 			}
 			debug_level = atoi(cp);
 		}
-		if (debug_level)
-			printf(_("Debug level set to %d.\n"), debug_level);
-		else
-			printf(_("Debug output turned off.\n"));
 		return true;
 	}
+#endif
 	if (strncasecmp(cp, "columns", 7) == 0) {
 		cp = skip_param(cp);
 		negate = (negate || isno(cp));
@@ -532,62 +517,45 @@ char	*cp;
 			}
 			screen_columns = atoi(cp);
 		}
-		if (screen_columns)
-			printf(_("Number of columns set to %d.\n"), screen_columns);
-		else
-			printf(_("Screen columns checking turned off.\n"));
 		return true;
 	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
 		negate = (negate || isno(cp));
-		if (negate) {
-			printf(_("Alphabetic case mode set to insensitive.\n"));
-			case_sensitive_flag = false;
-		} else {
-			printf(_("Alphabetic case mode set to sensitive.\n"));
-			case_sensitive_flag = true;
-		}
+		case_sensitive_flag = !negate;
 		return true;
 	}
 	if (strncasecmp(cp, "display2d", 7) == 0) {
 		cp = skip_param(cp);
 		negate = (negate || isno(cp));
-		if (negate) {
-			printf(_("Display mode set to single line format.\n"));
-			groupall = false;
-		} else {
-			printf(_("Display mode set to fraction format.\n"));
-			groupall = true;
-		}
+		groupall = !negate;
 		return true;
 	}
 	if (strncasecmp(cp, "preserve", 8) == 0) {
 		cp = skip_param(cp);
 		negate = (negate || isno(cp));
-		if (negate) {
-			printf(_("All roots will be approximated.\n"));
-			preserve_roots = false;
-		} else {
-			printf(_("Roots of rationals will be preserved so that they don't become irrational.\n"));
-			preserve_roots = true;
-		}
+		preserve_roots = !negate;
+		return true;
+	}
+	if (strncasecmp(cp, "true", 4) == 0) {
+		cp = skip_param(cp);
+		negate = (negate || isno(cp));
+		true_modulus = !negate;
 		return true;
 	}
 	if (strncasecmp(cp, "color", 5) == 0) {
 		cp = skip_param(cp);
 		negate = (negate || isno(cp));
-		if (negate) {
-			printf(_("ANSI color mode turned off.\n"));
-			color_flag = false;
-		} else {
-			printf(_("ANSI color mode turned on.\n"));
-			color_flag = true;
-		}
+		color_flag = !negate;
 		return true;
 	}
-	printf(_("Unknown option \"%s\".\n"), cp);
-	usage_flag = true;
+	if (strncasecmp(cp, "finance", 7) == 0) {
+		cp = skip_param(cp);
+		negate = (negate || isno(cp));
+		finance_option = !negate;
+		return true;
+	}
+	printf(_("Unknown set option \"%s\".\n"), cp);
 	return false;
 }
 
@@ -599,13 +567,15 @@ pause_cmd(cp)
 char	*cp;
 {
 	char	*cp1;
+	char	buf[100];
 
-	strcpy(prompt_str, _("Press the Enter key to continue or type \"quit\" to exit program..."));
-	if ((cp1 = getstring((char *) &tlhs[0], n_tokens * sizeof(token_type))) == NULL) {
+	strcpy(prompt_str, _(" ==== Please press Enter ==== "));
+	if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
 		return false;
 	}
-	if (strncasecmp(cp1, "quit", 4) == 0)
-		quit("");
+	if (strncasecmp(cp1, "quit", 4) == 0
+	    || strncasecmp(cp1, "abort", 5) == 0)
+		return false;
 	return true;
 }
 
@@ -636,9 +606,9 @@ char	*cp;
 	for (i1 = i; i1 <= j; i1++) {
 		if (exists[i1]) {
 			k = next_espace();
-			blt(&lhs[k][0], &lhs[i1][0], n_lhs[i1] * sizeof(token_type));
+			blt(lhs[k], lhs[i1], n_lhs[i1] * sizeof(token_type));
 			n_lhs[k] = n_lhs[i1];
-			blt(&rhs[k][0], &rhs[i1][0], n_rhs[i1] * sizeof(token_type));
+			blt(rhs[k], rhs[i1], n_rhs[i1] * sizeof(token_type));
 			n_rhs[k] = n_rhs[i1];
 			list_sub(k);
 		}
@@ -663,10 +633,10 @@ int	imag_flag;
 		return false;
 	}
 	j = next_espace();
-	uf_simp(&rhs[i][0], &n_rhs[i]);
-	factorv(&rhs[i][0], &n_rhs[i], (long) IMAGINARY);
+	uf_simp(rhs[i], &n_rhs[i]);
+	factorv(rhs[i], &n_rhs[i], (long) IMAGINARY);
 	partial_flag = false;
-	uf_simp(&rhs[i][0], &n_rhs[i]);
+	uf_simp(rhs[i], &n_rhs[i]);
 	partial_flag = true;
 	n_rhs[j] = 1;
 	rhs[j][0].level = 1;
@@ -708,16 +678,9 @@ int	imag_flag;
 		printf(_("No real parts found.  Equation is not complex.\n"));
 		return false;
 	}
-	blt(&lhs[j][0], &lhs[i][0], n_lhs[i] * sizeof(token_type));
+	blt(lhs[j], lhs[i], n_lhs[i] * sizeof(token_type));
 	n_lhs[j] = n_lhs[i];
-	printf(_("("));
-	list_proc(&lhs[j][0], n_lhs[j]);
-	if (imag_flag) {
-		printf(_(") is now equal to the imaginary part.\n"));
-	} else {
-		printf(_(") is now equal to the real part.\n"));
-	}
-	simp_divide(&rhs[j][0], &n_rhs[j], 0L);
+	simp_divide(rhs[j], &n_rhs[j], 0L);
 	cur_equation = j;
 	list_sub(cur_equation);
 	return true;
@@ -743,16 +706,21 @@ char	*cp;
 	return complex_sub(cp, true);
 }
 
+/*
+ * Do simplifications and approximations for the calculate command.
+ */
 calc_simp(equation, np)
 token_type	*equation;
 int		*np;
 {
+	in_calc_cmd = true;
 	subst_constants(equation, np);
 	simp_side(equation, np);
 	uf_simp(equation, np);
 	factorv(equation, np, (long) IMAGINARY);
 	simp_side(equation, np);
 	uf_simp(equation, np);
+	in_calc_cmd = false;
 }
 
 /*
@@ -772,10 +740,10 @@ char	*cp;
 	n_trhs = 1;
 	for (;;) {
 		printf(_("Running total = "));
-		list_proc(&trhs[0], n_trhs);
+		list_proc(trhs, n_trhs, false);
 		printf("\n");
 		strcpy(prompt_str, _("Enter value: "));
-		if (!get_expr(&tlhs[0], &n_tlhs)) {
+		if (!get_expr(tlhs, &n_tlhs)) {
 			break;
 		}
 		if ((n_trhs + 1 + n_tlhs) > n_tokens) {
@@ -791,9 +759,9 @@ char	*cp;
 		trhs[n_trhs].level = 1;
 		trhs[n_trhs].token.operatr = PLUS;
 		n_trhs++;
-		blt(&trhs[n_trhs], &tlhs[0], n_tlhs * sizeof(token_type));
+		blt(&trhs[n_trhs], tlhs, n_tlhs * sizeof(token_type));
 		n_trhs += n_tlhs;
-		calc_simp(&trhs[0], &n_trhs);
+		calc_simp(trhs, &n_trhs);
 	}
 	return true;
 }
@@ -812,19 +780,45 @@ char	*cp;
 	long		counter_max;
 	sign_array_type	sa_mark;
 	sign_array_type	sa_value;
-	jmp_buf		save_save;
-	int		factor_flag;
-	int		rv;
+	int		iterations;
+	long		it_v;
+	int		found;
+	int		n;
 
-	factor_flag = false;
-	if (strncasecmp(cp, "factor", 4) == 0) {
-		factor_flag = true;
-		cp = skip_param(cp);
-	}
-	if ((i = get_default_en(cp)) < 0)
+	iterations = 1;
+	it_v = 0;
+	i = cur_equation;
+	if (notdefined(i))
 		return false;
+	if (*cp) {
+		cp = parse_var2(&it_v, cp);
+		if (cp == NULL) {
+			return false;
+		}
+		iterations = atoi(cp);
+		if (iterations <= 0) {
+			printf(_("Invalid number of iterations.\n"));
+			return false;
+		}
+		cp = skip_num(cp);
+		if (extra_garbage(cp))
+			return false;
+	}
+	if (it_v) {
+		found = false;
+		for (j = 0; j < n_rhs[i]; j++) {
+			if (rhs[i][j].kind == VARIABLE) {
+				if (rhs[i][j].token.variable == it_v)
+					found = true;
+			}
+		}
+		if (!found) {
+			printf(_("Iteration variable not found in RHS.\n"));
+			return false;
+		}
+	}
 	n_trhs = n_rhs[i];
-	blt(&trhs[0], &rhs[i][0], n_trhs * sizeof(token_type));
+	blt(trhs, rhs[i], n_trhs * sizeof(token_type));
 	last_v = 0;
 	for (;;) {
 		v = -1;
@@ -838,12 +832,12 @@ char	*cp;
 		if (v == -1)
 			break;
 		last_v = v;
-		if ((v & VAR_MASK) == SIGN) {
+		if ((v & VAR_MASK) == SIGN || v == it_v) {
 			continue;
 		}
 		list_var(v, false, false);
 		sprintf(prompt_str, _("Enter %s: "), var_str);
-		if (!get_expr(&tlhs[0], &n_tlhs)) {
+		if (!get_expr(tlhs, &n_tlhs)) {
 			continue;
 		}
 		for (j = 0; j < n_tlhs; j++)
@@ -854,7 +848,24 @@ char	*cp;
 	for (j = 0; j < n_trhs; j += 2)
 		if (trhs[j].kind == VARIABLE && trhs[j].token.variable < 0)
 			trhs[j].token.variable = -trhs[j].token.variable;
-	simp_side(&trhs[0], &n_trhs);
+	if (it_v) {
+		list_var(it_v, false, false);
+		sprintf(prompt_str, _("Enter initial %s: "), var_str);
+		while (!get_expr(scratch, &n))
+			;
+		blt(tlhs, trhs, n_trhs * sizeof(token_type));
+		n_tlhs = n_trhs;
+		for (j = 0; j < iterations; j++) {
+			blt(trhs, tlhs, n_tlhs * sizeof(token_type));
+			n_trhs = n_tlhs;
+			subst_var_with_exp(trhs, &n_trhs, scratch, n, it_v);
+			calc_simp(trhs, &n_trhs);
+			blt(scratch, trhs, n_trhs * sizeof(token_type));
+			n = n_trhs;
+		}
+	} else {
+		simp_side(trhs, &n_trhs);
+	}
 	for (j = 0; j < ARR_CNT(sa_mark); j++)
 		sa_mark[j] = false;
 	for (j = 0; j < n_trhs; j += 2) {
@@ -869,15 +880,9 @@ char	*cp;
 		}
 	}
 	counter_max = (1L << k) - 1;
-	blt(save_save, jmp_save, sizeof(jmp_save));
 	counter = 0;
-	if ((rv = setjmp(jmp_save)) != 0) {
-		if (rv == 14)
-			printf(_("Expression too big!\n"));
-		counter++;
-	}
 	for (; counter <= counter_max; counter++) {
-		blt(&tlhs[0], &trhs[0], n_trhs * sizeof(token_type));
+		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
 		for (j = 0, k = 0; j < ARR_CNT(sa_mark); j++) {
 			if (sa_mark[j]) {
@@ -900,7 +905,7 @@ char	*cp;
 		for (j = 0, k = false; j < ARR_CNT(sa_mark); j++) {
 			if (sa_mark[j]) {
 				if (k)
-					fprintf(gfp, _(", "));
+					fprintf(gfp, ", ");
 				else {
 					fprintf(gfp, _("Solution #%ld with "), counter + 1);
 				}
@@ -915,18 +920,14 @@ char	*cp;
 			}
 		}
 		if (k)
-			fprintf(gfp, _(":\n"));
-		calc_simp(&tlhs[0], &n_tlhs);
-		if (factor_flag) {
-			factor_int(&tlhs[0], &n_tlhs);
-		}
+			fprintf(gfp, ":\n");
+		calc_simp(tlhs, &n_tlhs);
 		fprintf(gfp, " ");
-		list_proc(&lhs[i][0], n_lhs[i]);
+		list_proc(lhs[i], n_lhs[i], false);
 		fprintf(gfp, " = ");
-		list_proc(&tlhs[0], n_tlhs);
+		list_proc(tlhs, n_tlhs, false);
 		fprintf(gfp, "\n");
 	}
-	blt(jmp_save, save_save, sizeof(jmp_save));
 	return true;
 }
 
@@ -944,12 +945,10 @@ char	*cp;
 		for (i = 0; i < n_equations; i++) {
 			n_lhs[i] = 0;
 		}
-#if	LONG_VAR_NAMES
 		for (i = 0; var_names[i]; i++) {
 			free(var_names[i]);
 		}
 		var_names[0] = NULL;
-#endif
 	} else {
 		if (!get_range(&cp, &i, &j)) {
 			return false;
@@ -971,14 +970,14 @@ int	*diff_signp;
 	int	rv;
 
 	sign_flag = true;
-	rv = se_compare(&rhs[i][0], n_rhs[i], &rhs[i][0], n_rhs[i], diff_signp);
+	rv = se_compare(rhs[i], n_rhs[i], rhs[i], n_rhs[i], diff_signp);
 	sign_flag = false;
 	if (!rv || *diff_signp) {
 		printf(_("Error in compare function or too many terms to compare!\n"));
 		longjmp(jmp_save, 2);
 	}
 	sign_flag = true;
-	rv = se_compare(&rhs[i][0], n_rhs[i], &rhs[j][0], n_rhs[j], diff_signp);
+	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], diff_signp);
 	sign_flag = false;
 	return rv;
 }
@@ -1000,34 +999,34 @@ char	*cp;
 	if (notdefined(i)) {
 		return false;
 	}
-	if (strncasecmp(cp, "with", 4) == 0) {
+	if (strcmp_tospace(cp, "with") == 0) {
 		cp = skip_space(cp + 4);
 	}
 	if ((j = get_default_en(cp)) < 0) {
 		return false;
 	}
 	if (i == j) {
-		printf(_("Ridiculous command.\n"));
+		printf(_("Cannot compare an equation with itself.\n"));
 		return false;
 	}
 	printf(_("Comparing equation #%d with #%d...\n"), i + 1, j + 1);
 	already_solved = (solved_equation(i) && solved_equation(j));
 	if (already_solved) {
-		simp_loop(&rhs[i][0], &n_rhs[i]);
-		simp_loop(&rhs[j][0], &n_rhs[j]);
+		simp_loop(rhs[i], &n_rhs[i]);
+		simp_loop(rhs[j], &n_rhs[j]);
 		if (compare_rhs(i, j, &diff_sign)) {
 			goto times_neg1;
 		}
 		printf(_("Simplifying both equations...\n"));
-		simpa_side(&rhs[i][0], &n_rhs[i], false, false);
+		simpa_side(rhs[i], &n_rhs[i], false, false);
 		list_sub(i);
-		simpa_side(&rhs[j][0], &n_rhs[j], false, false);
+		simpa_side(rhs[j], &n_rhs[j], false, false);
 		list_sub(j);
 		if (compare_rhs(i, j, &diff_sign)) {
 			goto times_neg1;
 		}
-		uf_simp(&rhs[i][0], &n_rhs[i]);
-		uf_simp(&rhs[j][0], &n_rhs[j]);
+		uf_simp(rhs[i], &n_rhs[i]);
+		uf_simp(rhs[j], &n_rhs[j]);
 		if (compare_rhs(i, j, &diff_sign)) {
 			goto times_neg1;
 		}
@@ -1036,31 +1035,31 @@ char	*cp;
 	want.level = 1;
 	want.kind = CONSTANT;
 	want.token.constant = 0.0;
-	if (!solve_sub(&want, 1, &lhs[i][0], &n_lhs[i], &rhs[i][0], &n_rhs[i])
-	    || !solve_sub(&want, 1, &lhs[j][0], &n_lhs[j], &rhs[j][0], &n_rhs[j])) {
+	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])
+	    || !solve_sub(&want, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j])) {
 		printf(_("Can't solve for zero!\n"));
 		return false;
 	}
-	uf_simp(&rhs[i][0], &n_rhs[i]);
-	uf_simp(&rhs[j][0], &n_rhs[j]);
+	uf_simp(rhs[i], &n_rhs[i]);
+	uf_simp(rhs[j], &n_rhs[j]);
 	if (compare_rhs(i, j, &diff_sign)) {
 		printf(_("Equations are identical.\n"));
 		return true;
 	}
 	printf(_("Simplifying both equations...\n"));
-	simpa_side(&rhs[i][0], &n_rhs[i], false, false);
-	simpa_side(&rhs[j][0], &n_rhs[j], false, false);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
+	simpa_side(rhs[j], &n_rhs[j], false, false);
 	if (compare_rhs(i, j, &diff_sign)) {
 		printf(_("Equations are identical.\n"));
 		return true;
 	}
-	if (!solve_sub(&want, 1, &lhs[i][0], &n_lhs[i], &rhs[i][0], &n_rhs[i])
-	    || !solve_sub(&want, 1, &lhs[j][0], &n_lhs[j], &rhs[j][0], &n_rhs[j])) {
+	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])
+	    || !solve_sub(&want, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j])) {
 		printf(_("Can't solve for zero!\n"));
 		return false;
 	}
-	uf_simp(&rhs[i][0], &n_rhs[i]);
-	uf_simp(&rhs[j][0], &n_rhs[j]);
+	uf_simp(rhs[i], &n_rhs[i]);
+	uf_simp(rhs[j], &n_rhs[j]);
 	if (compare_rhs(i, j, &diff_sign)) {
 		printf(_("Equations are identical.\n"));
 		return true;
@@ -1073,12 +1072,12 @@ times_neg1:
 		return true;
 	}
 	printf(_("Variable ("));
-	list_proc(&lhs[i][0], n_lhs[i]);
+	list_proc(lhs[i], n_lhs[i], false);
 	printf(_(") in the first equation is equal to ("));
 	if (diff_sign) {
-		printf(_("-"));
+		printf("-");
 	}
-	list_proc(&lhs[j][0], n_lhs[j]);
+	list_proc(lhs[j], n_lhs[j], false);
 	printf(_(") in the second equation.\n"));
 	return(!diff_sign);
 }
@@ -1107,50 +1106,51 @@ char	*cp;
 	}
 	i = next_espace();
 	strcpy(prompt_str, _("Enter dividend: "));
-	if (!get_expr(&rhs[i][0], &nr)) {
+	if (!get_expr(rhs[i], &nr)) {
 		return false;
 	}
 	strcpy(prompt_str, _("Enter divisor: "));
-	if (!get_expr(&lhs[i][0], &nl)) {
+	if (!get_expr(lhs[i], &nl)) {
 		return false;
 	}
 	printf("\n");
-	simp_loop(&lhs[i][0], &nl);
-	simp_loop(&rhs[i][0], &nr);
-	uf_simp(&lhs[i][0], &nl);
-	uf_simp(&rhs[i][0], &nr);
+	simp_loop(lhs[i], &nl);
+	simp_loop(rhs[i], &nr);
+	uf_simp(lhs[i], &nl);
+	uf_simp(rhs[i], &nr);
 	if (nl == 1 && lhs[i][0].kind == CONSTANT
 	    && nr == 1 && rhs[i][0].kind == CONSTANT) {
 		d1 = rhs[i][0].token.constant;
 		d2 = lhs[i][0].token.constant;
 		d4 = modf(d1 / d2, &d3);
-		printf(_("Result of numerical division: %.14lg\n"), d1 / d2);
-		printf(_("Quotient: %.14lg, Remainder: %.14lg\n"), d3, d4 * d2);
-		return ngcd(d1, d2);
+		printf(_("Result of numerical division: %.14g\n"), d1 / d2);
+		printf(_("Quotient: %.14g, Remainder: %.14g\n"), d3, d4 * d2);
+		ngcd(d1, d2);
+		return true;
 	}
 	v_tmp = v;
-	if (poly_div(&rhs[i][0], nr, &lhs[i][0], nl, &v_tmp)) {
-		simp_divide(&tlhs[0], &n_tlhs, 0L);
-		simp_divide(&trhs[0], &n_trhs, 0L);
-		printf(_("Polynomial division successful using variable ("));
-		list_var(v_tmp, true, false);
-		printf(_(").\nThe quotient is:\n"));
-		list_proc(&tlhs[0], n_tlhs);
+	if (poly_div(rhs[i], nr, lhs[i], nl, &v_tmp)) {
+		simp_divide(tlhs, &n_tlhs, 0L);
+		simp_divide(trhs, &n_trhs, 0L);
+		list_var(v_tmp, false, false);
+		printf(_("Polynomial division successful using variable (%s).\n"), var_str);
+		printf(_("The quotient is:\n"));
+		list_proc(tlhs, n_tlhs, false);
 		printf(_("\n\nThe remainder is:\n"));
-		list_proc(&trhs[0], n_trhs);
+		list_proc(trhs, n_trhs, false);
 		printf("\n");
 	} else {
 		printf(_("Polynomial division failed.\n"));
 	}
 	printf("\n");
-	j = poly_gcd(&rhs[i][0], nr, &lhs[i][0], nl, v);
+	j = poly_gcd(rhs[i], nr, lhs[i], nl, v);
 	if (!j) {
-		j = poly_gcd(&lhs[i][0], nl, &rhs[i][0], nr, v);
+		j = poly_gcd(lhs[i], nl, rhs[i], nr, v);
 	}
 	if (j) {
-		simp_divide(&trhs[0], &n_trhs, 0L);
+		simp_divide(trhs, &n_trhs, 0L);
 		printf(_("Found polynomial Greatest Common Divisor (iterations = %d):\n"), j);
-		list_proc(&trhs[0], n_trhs);
+		list_proc(trhs, n_trhs, false);
 		printf("\n");
 	} else {
 		printf(_("No univariate polynomial GCD found.\n"));
@@ -1227,18 +1227,15 @@ next_var:
 	} else {
 		if (did_something) {
 			list_sub(cur_equation);
-		} else {
-			usage_flag = true;
 		}
 		return did_something;
 	}
 	if (!found_variable(cur_equation, v)) {
-		printf(_("Variable ("));
-		list_var(v, true, false);
-		printf(_(") not found in current equation.\n"));
+		list_var(v, false, false);
+		printf(_("Variable (%s) not found in current equation.\n"), var_str);
 		goto next_var;
 	}
-	using_flag = (strncasecmp(cp, "using", 5) == 0);
+	using_flag = (strcmp_tospace(cp, "using") == 0);
 	if (using_flag) {
 		cp = skip_space(cp + 5);
 		i = atoi(cp) - 1;
@@ -1247,16 +1244,18 @@ next_var:
 			return false;
 		}
 		if (i == cur_equation) {
-			printf(_("Ridiculous command.\n"));
+			printf(_("Error: source and destination equations are the same equation.\n"));
 			return false;
 		}
+		if (!elim_sub(i, v))
+			goto next_var;
 	} else {
+		n = 1;
 		i = cur_equation;
-		for (n = 1;; n++) {
+		for (;; n++) {
 			if (n >= n_equations) {
-				printf(_("Variable ("));
-				list_var(v, true, false);
-				printf(_(") not found in any other equation.\n"));
+				list_var(v, false, false);
+				printf(_("Variable (%s) not found in any other equation.\n"), var_str);
 				goto next_var;
 			}
 			if (i <= 0)
@@ -1265,12 +1264,12 @@ next_var:
 				i--;
 			if (used[i])
 				continue;
-			if (found_variable(i, v))
-				break;
+			if (found_variable(i, v)) {
+				if (elim_sub(i, v))
+					break;
+			}
 		}
 	}
-	if (!elim_sub(i, v))
-		goto next_var;
 	did_something = true;
 	simp_sub(cur_equation);
 	used[i] = true;
@@ -1357,14 +1356,17 @@ long	v;
 {
 	token_type	want;
 
-	printf(_("Solving equation #%d for ("), i + 1);
-	list_var(v, true, false);
-	printf(_(")...\n"));
+#if	!SILENT
+	list_var(v, false, false);
+	printf(_("Solving equation #%d for (%s)...\n"), i + 1, var_str);
+#endif
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
-	if (!solve_sub(&want, 1, &lhs[i][0], &n_lhs[i], &rhs[i][0], &n_rhs[i])) {
+	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])) {
+#if	!SILENT
 		printf(_("Solve failed.\n"));
+#endif
 		return false;
 	}
 	subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], rhs[i], n_rhs[i], v);
@@ -1386,7 +1388,7 @@ char	*cp;
 	int	factor_flag;
 
 	factor_flag = false;
-	if (strncasecmp(cp, "factor", 4) == 0) {
+	if (strcmp_tospace(cp, "factor") == 0) {
 		factor_flag = true;
 		cp = skip_param(cp);
 	}
@@ -1404,7 +1406,8 @@ char	*cp;
 		if (n_lhs[i] > 0) {
 			group_sub(i);
 			if (factor_flag) {
-				fi_sub(i);
+				factor_int(lhs[i], &n_lhs[i]);
+				factor_int(rhs[i], &n_rhs[i]);
 			}
 			flist_sub(i);
 		}
@@ -1421,25 +1424,23 @@ list_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	flag;
+	int	export_flag;
 
-	flag = false;
-	if (strncasecmp(cp, "export", 6) == 0) {
+	export_flag = false;
+	if (strcmp_tospace(cp, "export") == 0) {
 		cp = skip_param(cp);
-		flag = true;
+		export_flag = true;
 	}
 	if (!get_range(&cp, &i, &j)) {
 		return false;
 	}
 	if (extra_garbage(cp))
 		return false;
-	export_flag = flag;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
-			list1_sub(i);
+			list1_sub(i, export_flag);
 		}
 	}
-	export_flag = false;
 	return true;
 }
 
@@ -1456,12 +1457,12 @@ char	*cp;
 
 	java_flag = false;
 	int_flag = false;
-	if (strncasecmp(cp, "c", 1) == 0) {
+	if (strcmp_tospace(cp, "c") == 0) {
 		cp = skip_param(cp);
-	} else if (strncasecmp(cp, "java", 4) == 0) {
+	} else if (strcmp_tospace(cp, "java") == 0) {
 		cp = skip_param(cp);
 		java_flag = true;
-	} else if (strncasecmp(cp, "int", 3) == 0) {
+	} else if (strcmp_tospace(cp, "integer") == 0) {
 		cp = skip_param(cp);
 		int_flag = true;
 	}
@@ -1476,17 +1477,22 @@ char	*cp;
 				printf(_("LHS is not a variable in equation #%d.\n"), i + 1);
 				continue;
 			}
-			group_sub(i);
+			in_calc_cmd = true;
 			if (int_flag) {
-				if (!int_expr(lhs[i], n_lhs[i])
-				    || !int_expr(rhs[i], n_rhs[i])) {
-					printf(_("Equation #%d is not a listable integer equation.\n"), i + 1);
+				elim_loop(rhs[i], &n_rhs[i]);
+				uf_repeat_always(rhs[i], &n_rhs[i]);
+			}
+			group_sub(i);
+			in_calc_cmd = false;
+			if (int_flag) {
+				if (!int_expr(rhs[i], n_rhs[i])) {
+					printf(_("Equation #%d is not an integer equation.\n"), i + 1);
 					continue;
 				}
 			}
-			list_c_code(&lhs[i][0], n_lhs[i], java_flag, int_flag);
+			list_c_code(lhs[i], n_lhs[i], java_flag, int_flag);
 			fprintf(gfp, " = ");
-			list_c_code(&rhs[i][0], n_rhs[i], java_flag, int_flag);
+			list_c_code(rhs[i], n_rhs[i], java_flag, int_flag);
 			fprintf(gfp, ";\n\n");
 		}
 	}
@@ -1516,10 +1522,9 @@ char	*cp;
 		return false;
 	}
 	for (vc = 0; (cp != NULL) && *cp; vc++) {
-		if (strncasecmp(cp, "with", 4) == 0) {
+		if (strcmp_tospace(cp, "with") == 0) {
 			if (vc)
 				break;
-			usage_flag = true;
 			return false;
 		}
 		if (vc >= ARR_CNT(va)) {
@@ -1531,16 +1536,15 @@ char	*cp;
 			return false;
 		}
 		if (!found_variable(i, va[vc])) {
-			printf(_("Variable ("));
-			list_var(va[vc], true, false);
-			printf(_(") not found in current equation.\n"));
+			list_var(va[vc], false, false);
+			printf(_("Variable (%s) not found in current equation.\n"), var_str);
 			return false;
 		}
 	}
 	n_tlhs = n_lhs[i];
-	blt(&tlhs[0], &lhs[i][0], n_tlhs * sizeof(token_type));
+	blt(tlhs, lhs[i], n_tlhs * sizeof(token_type));
 	n_trhs = n_rhs[i];
-	blt(&trhs[0], &rhs[i][0], n_trhs * sizeof(token_type));
+	blt(trhs, rhs[i], n_trhs * sizeof(token_type));
 	last_v = 0;
 	for (;;) {
 		v = -1;
@@ -1571,8 +1575,7 @@ char	*cp;
 			if (!found)
 				continue;
 			if (*cp) {
-				if (strncasecmp(cp, "with", 4) != 0) {
-					usage_flag = true;
+				if (strcmp_tospace(cp, "with") != 0) {
 					return false;
 				}
 				cp1 = skip_space(cp + 4);
@@ -1580,9 +1583,8 @@ char	*cp;
 				if (!case_sensitive_flag) {
 					str_tolower(cp1);
 				}
-				if ((cp1 = parse_section(&scratch[0], &n, cp1)) == NULL
+				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL
 				    || n <= 0) {
-					usage_flag = true;
 					return false;
 				}
 				if (extra_garbage(cp1))
@@ -1592,7 +1594,7 @@ char	*cp;
 		}
 		list_var(v, false, false);
 		sprintf(prompt_str, _("Enter %s: "), var_str);
-		if (!get_expr(&scratch[0], &n)) {
+		if (!get_expr(scratch, &n)) {
 			continue;
 		}
 do_this:
@@ -1610,12 +1612,12 @@ do_this:
 	for (j = 0; j < n_trhs; j++)
 		if (trhs[j].kind == VARIABLE && trhs[j].token.variable < 0)
 			trhs[j].token.variable = -trhs[j].token.variable;
-	simp_loop(&tlhs[0], &n_tlhs);
-	simp_loop(&trhs[0], &n_trhs);
+	simp_loop(tlhs, &n_tlhs);
+	simp_loop(trhs, &n_trhs);
 	n_lhs[i] = n_tlhs;
-	blt(&lhs[i][0], &tlhs[0], n_tlhs * sizeof(token_type));
+	blt(lhs[i], tlhs, n_tlhs * sizeof(token_type));
 	n_rhs[i] = n_trhs;
-	blt(&rhs[i][0], &trhs[0], n_trhs * sizeof(token_type));
+	blt(rhs[i], trhs, n_trhs * sizeof(token_type));
 	list_sub(i);
 	return true;
 }
@@ -1665,16 +1667,18 @@ char	*cp;
 	}
 	i = next_espace();
 	er = !solved_equation(cur_equation);
-	er |= ((lhs[cur_equation][0].token.variable & VAR_MASK) <= SIGN
-	    || (lhs[cur_equation][0].token.variable & PERCENT_CHANGE));
+	if (!er) {
+		er = ((lhs[cur_equation][0].token.variable & VAR_MASK) <= SIGN
+		    || (lhs[cur_equation][0].token.variable & PERCENT_CHANGE));
+	}
 	if (er) {
 		printf(_("Please solve this equation for a normal variable and try again.\n"));
 		return false;
 	}
 	n = n_rhs[cur_equation];
-	blt(&rhs[i][0], &rhs[cur_equation][0], n * sizeof(token_type));
+	blt(rhs[i], rhs[cur_equation], n * sizeof(token_type));
 	n_rhs[i] = n;
-	blt(&lhs[i][0], &rhs[cur_equation][0], n * sizeof(token_type));
+	blt(lhs[i], rhs[cur_equation], n * sizeof(token_type));
 	for (j = n_rhs[i] - 1; j >= 0; j--) {
 		if (rhs[i][j].kind == VARIABLE) {
 			for (k = 0; k < vc; k++) {
@@ -1719,9 +1723,8 @@ char	*cp;
 	er = false;
 	for (j = 0; j < vc; j++) {
 		if (!fa[j]) {
-			printf(_("Variable ("));
-			list_var(va[j], true, false);
-			printf(_(") not found in RHS.\n"));
+			list_var(va[j], false, false);
+			printf(_("Variable (%s) not found in RHS.\n"), var_str);
 			er = true;
 		}
 	}
@@ -1761,7 +1764,7 @@ char	*cp;
 	ep->kind = CONSTANT;
 	ep->token.constant = 100.0;
 	n += 6;
-	if (!solve_sub(&want, 1, &lhs[i][0], &n, &rhs[i][0], &n_rhs[i])) {
+	if (!solve_sub(&want, 1, lhs[i], &n, rhs[i], &n_rhs[i])) {
 		printf(_("Solve failed!\n"));
 		return false;
 	}
@@ -1829,39 +1832,38 @@ char	*cp;
 	int	i1;
 	long	v;
 	double	d;
-	char	*cp1;
 
 	if (strncasecmp(cp, "number", 6) == 0) {
 		cp = skip_param(cp);
 next_number:
-		if (*cp) {
-			d = atof(cp);
-			cp = skip_param(cp);
-		} else {
-			strcpy(prompt_str, _("Enter integer to factor: "));
-			cp1 = getstring((char *) &scratch[0], n_tokens * sizeof(token_type));
-			if (cp1 == NULL)
+		if (*cp == '\0') {
+			strcpy(prompt_str, _("Enter integers to factor: "));
+			cp = getstring((char *) scratch, n_tokens * sizeof(token_type));
+			if (cp == NULL)
 				return false;
-			d = atof(cp1);
 		}
-		if (d == 0.0)
-			return false;
-		if (!factor_one_int(d)) {
-			printf(_("Not an integer.\n"));
+		d = strtod(cp, &cp);
+		cp = skip_space(cp);
+		if (!factor_one(d)) {
+			printf(_("Not a valid integer.\n"));
 			return false;
 		}
 		printf(_("%.0f = "), d);
 		for (i = 0; i < uno;) {
 			printf(_("%.0f"), unique[i]);
 			if (ucnt[i] > 1) {
-				printf(_("^%d"), ucnt[i]);
+				printf("^%d", ucnt[i]);
 			}
 			i++;
 			if (i < uno) {
-				printf(_(" * "));
+				printf(" * ");
 			}
 		}
 		printf("\n");
+		if (d != multiply_out_unique()) {
+			printf(_("Error in computation!\n"));
+			return false;
+		}
 		if (*cp)
 			goto next_number;
 		return true;
@@ -1878,8 +1880,8 @@ next_number:
 		}
 		for (i1 = i; i1 <= j; i1++) {
 			if (n_lhs[i1] > 0) {
-				simpv_side(&lhs[i1][0], &n_lhs[i1], v);
-				simpv_side(&rhs[i1][0], &n_rhs[i1], v);
+				simpv_side(lhs[i1], &n_lhs[i1], v);
+				simpv_side(rhs[i1], &n_rhs[i1], v);
 			}
 		}
 	} while (*cp);
@@ -1912,7 +1914,8 @@ char	*cp;
 	partial_flag = !fully_flag;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
-			unfa_sub(i);
+			uf_simp(lhs[i], &n_lhs[i]);
+			uf_simp(rhs[i], &n_rhs[i]);
 			list_sub(i);
 		}
 	}
@@ -1927,7 +1930,7 @@ int
 quit(cp)
 char	*cp;
 {
-	exit_program();
+	exit_program(0);
 	return false;
 }
 
@@ -1945,8 +1948,6 @@ char	*cp;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
-		printf(_("This command reads in the specified file as if you typed it in.\n"));
-		usage_flag = true;
 		return false;
 	}
 	fp = NULL;
@@ -1970,7 +1971,7 @@ char	*cp;
 		printf(_("Read operation aborted.\n"));
 		goto end_read;
 	}
-	while (cp = fgets((char *) &tlhs[0], n_tokens * sizeof(token_type), fp)) {
+	while (cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) {
 		input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
 		printf("%s", cp);
 		if (!process(cp)) {
@@ -1980,9 +1981,60 @@ char	*cp;
 end_read:
 	blt(jmp_save, save_save, sizeof(jmp_save));
 	fclose(fp);
-	usage_flag = false;
 	return(!rv);
 }
+
+/*
+ * Load $HOME/.mathomaticrc
+ *
+ * Return false if error encountered.
+ */
+int
+load_rc()
+{
+	int	i;
+	FILE	*fp;
+	char	buf[MAX_CMD_LEN];
+	char	*cp;
+	int	rv;
+
+#if	CYGWIN
+	strcpy(buf, dir_path);
+	strcat(buf, "/");
+	strcat(buf, RC_FILE);
+	fp = fopen(buf, "r");
+	if (fp == NULL) {
+		return true;
+	}
+#else
+	cp = getenv("HOME");
+	if (cp == NULL)
+		return true;
+	strcpy(buf, cp);
+	strcat(buf, "/");
+	strcat(buf, RC_FILE);
+	fp = fopen(buf, "r");
+	if (fp == NULL)
+		return true;
+#endif
+	rv = true;
+	while (cp = fgets(buf, sizeof(buf), fp)) {
+		i = strlen(cp) - 1;
+		while (i >= 0 && (cp[i] == '\n' || cp[i] == '\r')) {
+			cp[i--] = '\0';
+		}
+		for (i = 0; cp[i]; i++) {
+			if (cp[i] == ';') {
+				cp[i] = '\0';
+				break;
+			}
+		}
+		if (!set_options(cp))
+			rv = false;
+	}
+	fclose(fp);
+	return rv;
+}
 #endif
 
 #if	(UNIX || CYGWIN) && !SECURE
@@ -2004,16 +2056,10 @@ char	*cp;
 		}
 		gfp = fp;
 		high_prec = true;
-		list_cmd("all");
+		rv = list_cmd("all");
 		high_prec = false;
 		gfp = stdout;
-		rv = true;
-		if (fflush(fp) || ferror(fp)) {
-			rv = false;
-		}
-		if (fclose(fp))
-			rv = false;
-		if (!rv) {
+		if (fclose(fp)) {
 			printf(_("Error writing temporary file.\n"));
 			return false;
 		}
@@ -2022,7 +2068,6 @@ char	*cp;
 	} else {
 		if (access(cp, 6)) {
 			printf(_("You can only edit existing/writable files or all equations.\n"));
-			usage_flag = true;
 			return false;
 		}
 		return edit_sub(cp);
@@ -2033,7 +2078,7 @@ int
 edit_sub(cp)
 char	*cp;
 {
-	char	cl[500];
+	char	cl[MAX_CMD_LEN];
 	char	*cp1;
 	char	*editor_keyword = "EDITOR";
 
@@ -2058,8 +2103,9 @@ edit_again:
 	clear("all");
 	if (!read_in(cp)) {
 		printf(_("Prepare to run the editor.\n"));
-		pause_cmd("");
-		goto edit_again;
+		if (pause_cmd("")) {
+			goto edit_again;
+		}
 	}
 	unlink(tmp_file);
 	return true;
@@ -2079,14 +2125,12 @@ char	*cp;
 	int	rv;
 
 	if (*cp == '\0') {
-		printf(_("This command saves all equations in a file when followed by a filename.\n"));
-		usage_flag = true;
 		return false;
 	}
 	if (access(cp, 0) == 0) {
 		do {
 			sprintf(prompt_str, _("'%s' exists.  Overwrite (Y/N)? "), cp);
-			if ((cp1 = getstring((char *) &trhs[0], n_tokens * sizeof(token_type))) == NULL) {
+			if ((cp1 = getstring((char *) trhs, n_tokens * sizeof(token_type))) == NULL) {
 				return false;
 			}
 			str_tolower(cp1);
@@ -2106,9 +2150,6 @@ char	*cp;
 	rv = list_cmd("all");
 	high_prec = false;
 	gfp = stdout;
-	if (fflush(fp) || ferror(fp)) {
-		rv = false;
-	}
 	if (fclose(fp))
 		rv = false;
 	if (rv) {
@@ -2158,7 +2199,7 @@ int		*np;
 	char	*cp1;
 
 	for (;;) {
-		if ((cp1 = getstring(&buf[0], sizeof(buf) - 1)) == NULL) {
+		if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
 		if (!case_sensitive_flag) {
@@ -2184,7 +2225,7 @@ long	*vp;
 	char	*cp1;
 
 	strcpy(prompt_str, _("Enter variable: "));
-	if ((cp1 = getstring(&buf[0], sizeof(buf) - 1)) == NULL) {
+	if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
 		return false;
 	}
 	cp1 = parse_var2(vp, cp1);
@@ -2206,7 +2247,6 @@ int	i;
 {
 	if (i < 0 || i >= n_equations) {
 		printf(_("Invalid equation number.\n"));
-		usage_flag = true;
 		return true;
 	}
 	if (n_lhs[i] <= 0) {
