@@ -7,6 +7,8 @@
 #include "am.h"
 #include "externs.h"
 
+static int	d_recurse();
+
 /*
  * Compute the derivative of an equation side, with respect to the variable "v",
  * using the fast transform method.
@@ -45,7 +47,7 @@ long		v;
  * Return "true" if successful, "false" if it is beyond this program's
  * capabilities or an error was encountered.
  */
-int
+static int
 d_recurse(equation, np, loc, level, v)
 token_type	*equation;
 int		*np, loc, level;
@@ -56,7 +58,7 @@ long		v;
 	int	op;
 	int	oploc;
 	int	endloc;
-	double	d;
+	complexs	c;
 
 	if (equation[loc].level < level) {
 /* First differentiate if it is a single variable or constant. */
@@ -200,18 +202,20 @@ d_power:
 		    && ((v == MATCH_ANY && (equation[i].token.variable & VAR_MASK) > SIGN)
 		    || equation[i].token.variable == v)) {
 			if ((oploc - loc) == 1) {
+				c.im = 0.0;
 				switch (equation[loc].kind) {
 				case CONSTANT:
-					d = equation[loc].token.constant;
+					c.re = equation[loc].token.constant;
 					break;
 				case VARIABLE:
-					if (!var_is_const(equation[loc].token.variable, &d))
+					if (!var_is_const(equation[loc].token.variable, &c.re))
 						return false;
 					break;
 				default:
 					return false;
 				}
-				n = (endloc - oploc) + 2;
+				c = complex_log(c);
+				n = (endloc - oploc) + 6;
 				if (*np + n > n_tokens) {
 					error_huge();
 				}
@@ -222,9 +226,25 @@ d_power:
 				equation[n].kind = OPERATOR;
 				equation[n].token.operatr = TIMES;
 				n++;
-				equation[n].level = level;
+				equation[n].level = level + 1;
 				equation[n].kind = CONSTANT;
-				equation[n].token.constant = log(d);
+				equation[n].token.constant = c.re;
+				n++;
+				equation[n].level = level + 1;
+				equation[n].kind = OPERATOR;
+				equation[n].token.operatr = PLUS;
+				n++;
+				equation[n].level = level + 2;
+				equation[n].kind = CONSTANT;
+				equation[n].token.constant = c.im;
+				n++;
+				equation[n].level = level + 2;
+				equation[n].kind = OPERATOR;
+				equation[n].token.operatr = TIMES;
+				n++;
+				equation[n].level = level + 2;
+				equation[n].kind = VARIABLE;
+				equation[n].token.variable = IMAGINARY;
 				n++;
 				equation[n].level = level;
 				equation[n].kind = OPERATOR;
@@ -237,12 +257,11 @@ d_power:
 				if (!d_recurse(equation, np, n, level + 1, v))
 					return false;
 				return true;
-			} else {
-				return false;
 			}
+			return false;
 		}
 	}
-	blt(&scratch[0], &equation[oploc+1], (endloc - (oploc + 1)) * sizeof(token_type));
+	blt(scratch, &equation[oploc+1], (endloc - (oploc + 1)) * sizeof(token_type));
 	n = endloc - (oploc + 1);
 	scratch[n].level = level;
 	scratch[n].kind = OPERATOR;
@@ -282,7 +301,7 @@ d_power:
 	}
 	blt(&equation[loc+n], &equation[endloc], (*np - endloc) * sizeof(token_type));
 	*np += loc + n - endloc;
-	blt(&equation[loc], &scratch[0], n * sizeof(token_type));
+	blt(&equation[loc], scratch, n * sizeof(token_type));
 	if (!d_recurse(equation, np, loc + j, level + 1, v))
 		return false;
 	return true;
@@ -317,7 +336,7 @@ char	*cp;
 				return false;
 			}
 		}
-		if (isascii(*cp) && isdigit(*cp)) {
+		if (*cp) {
 			order = atoi(cp);
 			cp = skip_num(cp);
 		}
@@ -327,7 +346,6 @@ char	*cp;
 	}
 	if (order < 1) {
 		printf(_("Order out of range.\n"));
-		usage_flag = true;
 		return false;
 	}
 	if (v == 0) {
@@ -338,7 +356,9 @@ char	*cp;
 	i = next_espace();
 	found = false;
 	er = !solved_equation(cur_equation);
-	er |= ((lhs[cur_equation][0].token.variable & VAR_MASK) <= SIGN);
+	if (!er) {
+		er = ((lhs[cur_equation][0].token.variable & VAR_MASK) <= SIGN);
+	}
 	for (j = 0; j < n_rhs[cur_equation]; j += 2) {
 		if (rhs[cur_equation][j].kind == VARIABLE) {
 			if (rhs[cur_equation][j].token.variable == v)
@@ -351,19 +371,15 @@ char	*cp;
 	}
 	blt(rhs[i], rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
 	n_rhs[i] = n_rhs[cur_equation];
-	simp_loop(&rhs[i][0], &n_rhs[i]);
+	simp_loop(rhs[i], &n_rhs[i]);
 	for (j = 0; j < order; j++) {
-		if (!differentiate(&rhs[i][0], &n_rhs[i], v)) {
+		if (!differentiate(rhs[i], &n_rhs[i], v)) {
 			printf(_("Differentiation failed.\n"));
 			return false;
 		}
-#if	true
 		simpa_side(rhs[i], &n_rhs[i], false, true);
-#else
-		simp_loop(&rhs[i][0], &n_rhs[i]);
-#endif
 	}
-	blt(&lhs[i][0], &lhs[cur_equation][0], n_lhs[cur_equation] * sizeof(token_type));
+	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 	n_lhs[i] = n_lhs[cur_equation];
 	if (!er && order < 10) {
 /* increment the number of primes in the LHS variable */
@@ -382,6 +398,86 @@ char	*cp;
 	return true;
 }
 
+/*
+ * The extrema command.
+ */
+int
+extrema_cmd(cp)
+char	*cp;
+{
+	long	v;
+	int	i, j;
+	int	found;
+	token_type	want;
+
+	if (notdefined(cur_equation)) {
+		return false;
+	}
+	if (!solved_equation(cur_equation)) {
+		printf(_("Current equation is not solved for a variable.\n"));
+		return false;
+	}
+	if (*cp) {
+		cp = parse_var2(&v, cp);
+		if (cp == NULL) {
+			return false;
+		}
+	} else {
+		if (!prompt_var(&v)) {
+			return false;
+		}
+	}
+	if (extra_garbage(cp)) {
+		return false;
+	}
+	i = next_espace();
+	found = false;
+	for (j = 0; j < n_rhs[cur_equation]; j += 2) {
+		if (rhs[cur_equation][j].kind == VARIABLE) {
+			if (rhs[cur_equation][j].token.variable == v)
+				found = true;
+		}
+	}
+	if (!found) {
+		printf(_("Variable not found in RHS.\n"));
+		return false;
+	}
+	blt(rhs[i], rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
+	n_rhs[i] = n_rhs[cur_equation];
+	simp_loop(rhs[i], &n_rhs[i]);
+	if (!differentiate(rhs[i], &n_rhs[i], v)) {
+		printf(_("Differentiation failed.\n"));
+		return false;
+	}
+	simpa_side(rhs[i], &n_rhs[i], false, true);
+	found = false;
+	for (j = 0; j < n_rhs[i]; j += 2) {
+		if (rhs[i][j].kind == VARIABLE) {
+			if (rhs[i][j].token.variable == v)
+				found = true;
+		}
+	}
+	if (!found) {
+		printf(_("There are no extrema for this equation.\n"));
+		return false;
+	}
+	n_lhs[i] = 1;
+	lhs[i][0].level = 1;
+	lhs[i][0].kind = CONSTANT;
+	lhs[i][0].token.constant = 0.0;
+	cur_equation = i;
+	want.level = 1;
+	want.kind = VARIABLE;
+	want.token.variable = v;
+	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])) {
+		printf(_("Solve failed.\n"));
+		return false;
+	}
+	simpa_side(rhs[i], &n_rhs[i], false, false);
+	list_sub(cur_equation);
+	return true;
+}
+
 /*
  * The taylor command.
  */
@@ -417,7 +513,7 @@ char	*cp;
 			return false;
 	}
 	i = next_espace();
-	blt(&lhs[i][0], &lhs[cur_equation][0], n_lhs[cur_equation] * sizeof(token_type));
+	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 	n_lhs[i] = n_lhs[cur_equation];
 	n_rhs[i] = 0;
 	our = next_espace();
@@ -433,26 +529,27 @@ char	*cp;
 		printf(_("Variable not found in RHS.\n"));
 		return false;
 	}
-	printf(_("Taylor approximation of current equation about "));
-	list_var(v, true, false);
-	printf(_(" = point.\n"));
+	list_var(v, false, false);
+	printf(_("Taylor approximation of current equation about %s = point.\n"), var_str);
 	blt(rhs[our], rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
 	our_nrhs = n_rhs[cur_equation];
-	if (!differentiate(&rhs[our][0], &our_nrhs, v)) {
+	if (!differentiate(rhs[our], &our_nrhs, v)) {
 		printf(_("Differentiation failed.\n"));
 		return false;
 	}
+oops:
 	strcpy(prompt_str, _("Enter point: "));
-	if (!get_expr(&lhs[our][0], &our_nlhs)) {
-		return false;
+	if (!get_expr(lhs[our], &our_nlhs)) {
+		printf(_("Enter 0 if you don't know.\n"));
+		goto oops;
 	}
 oops2:
 	strcpy(prompt_str, _("Enter order (number of derivatives to take): "));
-	if ((cp1 = getstring((char *) &scratch[0], n_tokens * sizeof(token_type))) == NULL)
+	if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
 		return false;
-	if (*cp1 == '\0' || *cp1 == '\n') {
+	if (*cp1 == '\0') {
 		printf(_("Derivatives will be taken until they reach zero.\n"));
-		order = 32000;
+		order = INT_MAX;
 	} else {
 		if (!isascii(*cp1) || !isdigit(*cp1))
 			goto oops2;
@@ -463,7 +560,7 @@ oops2:
 	}
 	n = 0;
 	i1 = 0;
-	blt(&rhs[i][0], &rhs[cur_equation][0], n_rhs[cur_equation] * sizeof(token_type));
+	blt(rhs[i], rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
 	n_rhs[i] = n_rhs[cur_equation];
 loop_again:
 	for (k = i1; k < n_rhs[i]; k += 2) {
@@ -474,7 +571,7 @@ loop_again:
 			blt(&rhs[i][k+our_nlhs], &rhs[i][k+1], (n_rhs[i] - (k + 1)) * sizeof(token_type));
 			n_rhs[i] += our_nlhs - 1;
 			j1 = k;
-			blt(&rhs[i][k], &lhs[our][0], our_nlhs * sizeof(token_type));
+			blt(&rhs[i][k], lhs[our], our_nlhs * sizeof(token_type));
 			k += our_nlhs;
 			for (; j1 < k; j1++)
 				rhs[i][j1].level += level;
@@ -500,7 +597,7 @@ loop_again:
 	ep->token.operatr = MINUS;
 	k += 3;
 	j1 = k;
-	blt(&rhs[i][k], &lhs[our][0], our_nlhs * sizeof(token_type));
+	blt(&rhs[i][k], lhs[our], our_nlhs * sizeof(token_type));
 	k += our_nlhs;
 	for (; j1 < k; j1++)
 		rhs[i][j1].level += 3;
@@ -526,15 +623,15 @@ loop_again:
 	n_rhs[i] = k;
 	for (; i1 < k; i1++)
 		rhs[i][i1].level++;
-	simp_loop(&rhs[i][0], &n_rhs[i]);
+	simp_loop(rhs[i], &n_rhs[i]);
 	if (n < order) {
 		if (n > 0) {
-			if (!differentiate(&rhs[our][0], &our_nrhs, v)) {
+			if (!differentiate(rhs[our], &our_nrhs, v)) {
 				printf(_("Differentiation failed.\n"));
 				return false;
 			}
 		}
-		simpa_side(&rhs[our][0], &our_nrhs, false, true);
+		simpa_side(rhs[our], &our_nrhs, false, true);
 		side_debug(3, rhs[our], our_nrhs);
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT
 		    || rhs[our][0].token.constant != 0.0) {
@@ -548,7 +645,7 @@ loop_again:
 			rhs[i][k].token.operatr = PLUS;
 			k++;
 			i1 = k;
-			blt(&rhs[i][k], &rhs[our][0], our_nrhs * sizeof(token_type));
+			blt(&rhs[i][k], rhs[our], our_nrhs * sizeof(token_type));
 			n_rhs[i] = k + our_nrhs;
 			n++;
 			goto loop_again;
@@ -559,3 +656,154 @@ loop_again:
 	list_sub(cur_equation);
 	return true;
 }
+
+/*
+ * The limit command.
+ */
+int
+limit_cmd(cp)
+char	*cp;
+{
+	int		i, j, k;
+	long		v;
+	token_type	solved_v, want;
+	char		*cp_start;
+	int		n;
+	int		infinity_flag;
+	int		found, found_den;
+	int		num, den;
+	int		complex_den;
+	int		level;
+
+	cp_start = cp;
+	if (notdefined(cur_equation)) {
+		return false;
+	}
+	if (!solved_equation(cur_equation)) {
+		printf(_("Current equation is not solved for a variable.\n"));
+		return false;
+	}
+	blt(&solved_v, lhs[cur_equation], sizeof(token_type));
+	i = next_espace();
+	if (*cp) {
+		cp = parse_var2(&v, cp);
+		if (cp == NULL) {
+			return false;
+		}
+	} else {
+		if (!prompt_var(&v)) {
+			return false;
+		}
+	}
+	found = false;
+	for (j = 0; j < n_rhs[cur_equation]; j += 2) {
+		if (rhs[cur_equation][j].kind == VARIABLE) {
+			if (rhs[cur_equation][j].token.variable == v)
+				found = true;
+		}
+	}
+	if (!found) {
+		printf(_("Variable not found in RHS.\n"));
+		return false;
+	}
+	want.level = 1;
+	want.kind = VARIABLE;
+	want.token.variable = v;
+	if (*cp) {
+		input_column += (cp - cp_start);
+		if (!case_sensitive_flag) {
+			str_tolower(cp);
+		}
+		if ((cp = parse_section(lhs[i], &n, cp)) == NULL || n <= 0) {
+			return false;
+		}
+		if (extra_garbage(cp))
+			return false;
+	} else {
+		strcpy(prompt_str, _("Enter expression: "));
+		if (!get_expr(lhs[i], &n)) {
+			return false;
+		}
+	}
+	simp_loop(lhs[i], &n);
+	infinity_flag = (n == 1 && lhs[i][0].kind == CONSTANT
+	    && isinf(lhs[i][0].token.constant));
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
+	found_den = false;
+	complex_den = false;
+	for (j = 0; j < n_rhs[cur_equation]; j++) {
+		switch (rhs[cur_equation][j].kind) {
+		case OPERATOR:
+			switch (rhs[cur_equation][j].token.operatr) {
+			case DIVIDE:
+			case MODULUS:
+				num = false;
+				den = false;
+				level = rhs[cur_equation][j].level;
+				for (k = j + 2;; k += 2) {
+					if (rhs[cur_equation][k-1].kind == VARIABLE
+					    && rhs[cur_equation][k-1].token.variable == v) {
+						den = true;
+						found_den = true;
+					}
+					if (k >= n_rhs[cur_equation] || rhs[cur_equation][k].level <= level) {
+						break;
+					}
+					switch (rhs[cur_equation][k].token.operatr) {
+					case DIVIDE:
+					case MODULUS:
+						complex_den = true;
+						break;
+					}
+				}
+				for (k = j - 1; k >= 0 && rhs[cur_equation][k].level >= level; k--) {
+					if (rhs[cur_equation][k].kind == VARIABLE
+					    && rhs[cur_equation][k].token.variable == v) {
+						num = true;
+					}
+				}
+				if (num && den) {
+					complex_den = true;
+				}
+			}
+			break;
+		}
+	}
+	if (!found_den || (infinity_flag && !complex_den)) {
+		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], lhs[i], n, v);
+		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+		list_sub(cur_equation);
+		return true;
+	}
+	if (infinity_flag) {
+		lhs[i][0].token.constant = 0.0;
+		tlhs[0].level = 1;
+		tlhs[0].kind = CONSTANT;
+		tlhs[0].token.constant = 1.0;
+		tlhs[1].level = 1;
+		tlhs[1].kind = OPERATOR;
+		tlhs[1].token.operatr = DIVIDE;
+		tlhs[2].level = 1;
+		tlhs[2].kind = VARIABLE;
+		tlhs[2].token.variable = v;
+		n_tlhs = 3;
+		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tlhs, n_tlhs, v);
+	}
+	if (!solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation])) {
+		printf(_("Can't take limit of this equation because solve failed.\n"));
+		return false;
+	}
+	blt(lhs[cur_equation], lhs[i], n * sizeof(token_type));
+	n_lhs[cur_equation] = n;
+	symb_flag = true;
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	symb_flag = false;
+	if (!solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation])) {
+		printf(_("Can't take limit of this equation because solve failed.\n"));
+		return false;
+	}
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	list_sub(cur_equation);
+	return true;
+}
