@@ -20,16 +20,16 @@ extern token_type	*trhs;
 
 extern token_type	*scratch;
 
-#if	LONG_VAR_NAMES
 extern char		*var_names[MAX_VAR_NAMES];
-#endif
 
 extern int		case_sensitive_flag;
 extern int		groupall;
+extern int		in_calc_cmd;
 extern int		preserve_roots;
-extern int		export_flag;
+extern int		true_modulus;
 extern int		screen_columns;
 extern int		screen_rows;
+extern int		finance_option;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
@@ -39,13 +39,13 @@ extern int		symb_flag;
 extern int		high_prec;
 extern int		input_column;
 extern int		sign_flag;
-extern int		usage_flag;
 extern int		piping_in_flag;
 extern char		*tmp_file;
 extern double		small_epsilon;
 extern double		epsilon;
 extern char		var_str[];
 extern char		prompt_str[];
+extern char		*dir_path;
 
 extern double		unique[];
 extern int		ucnt[];
@@ -56,3 +56,5 @@ extern sign_array_type	sign_array;
 extern FILE		*gfp;
 
 extern jmp_buf		jmp_save;
+
+extern int		test_mode;
