@@ -10,8 +10,18 @@
 #include "am.h"
 #include "externs.h"
 
-double nn, vv;
-double sq[] = {
+static void	try_factor();
+static int	fplus_recurse();
+static int	fplus_sub();
+static int	big_fplus();
+static int	ftimes_recurse();
+static int	ftimes_sub();
+static int	fpower_recurse();
+static int	fpower_sub();
+static int	fc_recurse();
+
+static double nn, vv;
+static double sq[] = {
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
 	 2, 6, 4, 6, 8, 4, 2, 4,
@@ -26,7 +36,7 @@ double sq[] = {
  * Return true if successful.
  */
 int
-factor_one_int(start)
+factor_one(start)
 double	start;
 {
 	int		i;
@@ -36,7 +46,7 @@ double	start;
 	nn = start;
 	if (nn == 0.0)
 		return false;
-	if (fabs(nn) > 1.0e14) {
+	if (fabs(nn) >= MAX_K_INTEGER) {
 		/* too large to factor */
 		return false;
 	}
@@ -57,7 +67,7 @@ double	start;
 		}
 	}
 	if (fmod(nn, 1.0) != 0.0) {
-		printf(_("Floating point error in factor_one_int()!\n"));
+		printf(_("Floating point error in factor_one()!\n"));
 		return false;
 	}
 	if (nn != 1.0) {
@@ -70,6 +80,7 @@ double	start;
  * See if "arg" is one or more factors of "nn".
  * If so, save it and remove it from "nn".
  */
+static void
 try_factor(arg)
 double	arg;
 {
@@ -82,7 +93,7 @@ double	arg;
 				ucnt[uno] = 1;
 				uno++;
 			}
-			nn = nn/arg;
+			nn /= arg;
 			vv = 1.0 + sqrt(fabs(nn));
 			if (nn <= 1.0 && nn >= -1.0)
 				break;
@@ -92,6 +103,24 @@ double	arg;
 	}
 }
 
+/*
+ * Convert unique[] back into an integer.
+ */
+double
+multiply_out_unique()
+{
+	int	i, j;
+	double	d;
+
+	d = 1.0;
+	for (i = 0; i < uno; i++) {
+		for (j = 0; j < ucnt[i]; j++) {
+			d *= unique[i];
+		}
+	}
+	return d;
+}
+
 /*
  * Factor integers in an equation side.
  */
@@ -105,7 +134,7 @@ int		*np;
 
 	for (i = 0; i < *np; i += 2) {
 		if (equation[i].kind == CONSTANT) {
-			if (factor_one_int(equation[i].token.constant) && uno > 0) {
+			if (factor_one(equation[i].token.constant) && uno > 0) {
 				level = equation[i].level;
 				if (uno > 1 && *np > 1)
 					level++;
@@ -156,13 +185,6 @@ int		*np;
 	}
 }
 
-fi_sub(n)
-int	n;
-{
-	factor_int(&lhs[n][0], &n_lhs[n]);
-	factor_int(&rhs[n][0], &n_rhs[n]);
-}
-
 /*
  * Factor divides only.
  * (a/c + b/c) -> ((a+b)/c).
@@ -219,7 +241,7 @@ double		d;
 	return fplus_recurse(equation, np, 0, 1, v, d, 0);
 }
 
-int
+static int
 fplus_recurse(equation, np, loc, level, v, d, factor_code)
 token_type	*equation;
 int		*np, loc, level;
@@ -289,7 +311,7 @@ skip_plus:
 	return modified;
 }
 
-int
+static int
 fplus_sub(equation, np, loc, i1, n1, i2, n2, level, whole_flag, div_only, v, d)
 token_type	*equation;
 int		*np, loc, i1, n1, i2, n2;
@@ -307,7 +329,7 @@ double		d;
 	int	ai, aj;
 	int	flag1, flag2;
 	int	same_flag;
-	k_type	save_k1, save_k2;
+	double	save_k1, save_k2;
 	double	save_d1, save_d2;
 	double	power;
 	double	d1, d2;
@@ -578,7 +600,7 @@ end_mess:
 		*np -= n2 + 1;
 		blt(&equation[i1+len], &equation[e1], (*np - e1) * sizeof(*equation));
 		*np += len - n1;
-		blt(&equation[i1], &scratch[0], len * sizeof(*equation));
+		blt(&equation[i1], scratch, len * sizeof(*equation));
 		return true;
 	}
 	if (whole_flag)
@@ -688,7 +710,7 @@ end_mess:
 	goto f_inner;
 }
 
-int
+static int
 big_fplus(equation, level, diff_sign, sop1, op1, op2, i1, i2, b1, b2, ai, aj, i, j, e1, e2)
 token_type	*equation;
 int		level;
@@ -811,7 +833,7 @@ int		*np;
 	return ftimes_recurse(equation, np, 0, 1);
 }
 
-int
+static int
 ftimes_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
@@ -878,7 +900,7 @@ skip_times:
 	return modified;
 }
 
-int
+static int
 ftimes_sub(equation, np, loc, i1, n1, i2, n2, level)
 token_type	*equation;
 int		*np, loc, i1, n1, i2, n2, level;
@@ -890,20 +912,25 @@ int		*np, loc, i1, n1, i2, n2, level;
 	int	diff_sign;
 	int	both_divide;
 
-	if ((n1 == 1 && equation[i1].kind == CONSTANT)
-	    && (n2 == 1 && equation[i2].kind == CONSTANT))
-		return false;
+	op2 = equation[i2-1].token.operatr;
 	e1 = i1 + n1;
 	e2 = i2 + n2;
-	op2 = equation[i2-1].token.operatr;
 	if (i1 <= loc) {
 		op1 = TIMES;
-	} else
+	} else {
 		op1 = equation[i1-1].token.operatr;
-#if	true
+	}
+	if ((n1 == 1 && equation[i1].kind == CONSTANT)
+	    || (n2 == 1 && equation[i2].kind == CONSTANT)) {
+		return false;
+	}
 	both_divide = (op1 == DIVIDE && op2 == DIVIDE);
-#else
-	both_divide = false;
+#if	true
+	if (se_compare(&equation[i1], e1 - i1, &equation[i2], e2 - i2, &diff_sign)) {
+		i = e1;
+		j = e2;
+		goto common_base;
+	}
 #endif
 	for (i = i1 + 1; i < e1; i += 2) {
 		if (equation[i].level == level
@@ -938,6 +965,8 @@ common_base:
 	    + rlen1 + 1
 	    + ((j == e2) ? 1 : (e2 - j - 1));
 	len -= n1;
+	if (j - i2 == 1 && equation[i2].kind == CONSTANT && equation[i2].token.constant == -1.0)
+		return false;
 	if (diff_sign) {
 		if (j - i2 == 1 && equation[i2].kind == CONSTANT)
 			return false;
@@ -1009,17 +1038,14 @@ common_base:
  * Return true if equation side was modified.
  */
 int
-fctor_power(equation, np)
+factor_power(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	rv;
-
-	rv = fpower_recurse(equation, np, 0, 1);
-	return rv;
+	return fpower_recurse(equation, np, 0, 1);
 }
 
-int
+static int
 fpower_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
@@ -1068,10 +1094,6 @@ f_again:
 		if (i >= *np || equation[i-1].level < level)
 			break;
 	}
-#if	false
-	if (modified)
-		return true;
-#endif
 skip_power:
 	for (i = loc;;) {
 		if (i >= *np || equation[i].level < level)
@@ -1088,7 +1110,7 @@ skip_power:
 	return modified;
 }
 
-int
+static int
 fpower_sub(equation, np, loc, i1, n1, i2, n2, level)
 token_type	*equation;
 int		*np, loc, i1, n1, i2, n2, level;
@@ -1135,6 +1157,9 @@ fp_outer:
 	b1 = i + 1;
 	if (b1 >= e1)
 		return false;
+#if	false
+	i = e1;
+#else
 	i = b1 + 1;
 	for (; i < e1; i += 2) {
 		if (equation[i].level == (level + 1)
@@ -1143,6 +1168,7 @@ fp_outer:
 			break;
 		}
 	}
+#endif
 	temp.level = 1;
 	temp.kind = CONSTANT;
 	temp.token.constant = 1.0;
@@ -1154,6 +1180,9 @@ fp_inner:
 	b2 = j + 1;
 	if (b2 >= e2)
 		goto fp_outer;
+#if	false
+	j = e2;
+#else
 	j = b2 + 1;
 	for (; j < e2; j += 2) {
 		if (equation[j].level == (level + 1)
@@ -1162,6 +1191,7 @@ fp_inner:
 			break;
 		}
 	}
+#endif
 	if (equation[b2-1].token.operatr == POWER) {
 		if (pop1 != TIMES)
 			goto fp_inner;
@@ -1235,7 +1265,7 @@ fp_inner:
 		*np -= n2 + 1;
 		blt(&equation[i1+len], &equation[e1], (*np - e1) * sizeof(*equation));
 		*np += len - n1;
-		blt(&equation[i1], &scratch[0], len * sizeof(*equation));
+		blt(&equation[i1], scratch, len * sizeof(*equation));
 		return true;
 	}
 	goto fp_inner;
@@ -1274,7 +1304,7 @@ int		level_code;
 	return fc_recurse(equation, np, 0, 1, level_code);
 }
 
-int
+static int
 fc_recurse(equation, np, loc, level, level_code)
 token_type	*equation;
 int		*np, loc, level;
@@ -1402,9 +1432,12 @@ break_cont:
 					d = equation[i].token.constant;
 				}
 			}
-			if (minimum < 1.0 && fmod(d, 1.0) == 0.0)
-				return modified;
-			if (fmod(d / minimum, 1.0) != 0.0) {
+			if ((minimum < 1.0 && fmod(d, 1.0) == 0.0)
+			    || (fmod(d / minimum, 1.0) != 0.0)) {
+				if (neg_flag) {
+					minimum = 1.0;
+					break;
+				}
 				return modified;
 			}
 			i++;
