@@ -9,19 +9,28 @@
 
 #define	MAX_COMPARE_TERMS	200	/* Max no. of terms on same level that can be compared. */
 
+static int	sp2_sub();
+static int	sp_sub();
+static int	org_recurse();
+static void	org_up_level();
+static int	const_recurse();
+static int	compare_recurse();
+static int	order_recurse();
+static void	swap();
+
 simp_sub(n)
 int	n;
 {
-	simp_loop(&lhs[n][0], &n_lhs[n]);
-	simp_loop(&rhs[n][0], &n_rhs[n]);
+	simp_loop(lhs[n], &n_lhs[n]);
+	simp_loop(rhs[n], &n_rhs[n]);
 }
 
 simpa_sub(n, poly_flag, quick_flag)
 int	n;
 int	poly_flag, quick_flag;
 {
-	simpa_side(&lhs[n][0], &n_lhs[n], poly_flag, quick_flag);
-	simpa_side(&rhs[n][0], &n_rhs[n], poly_flag, quick_flag);
+	simpa_side(lhs[n], &n_lhs[n], poly_flag, quick_flag);
+	simpa_side(rhs[n], &n_rhs[n], poly_flag, quick_flag);
 }
 
 simp_side(equation, np)
@@ -67,6 +76,7 @@ int		*np;
 	int	i;
 	int	level;
 
+	simp_loop(equation, np);
 	for (i = 0; i < *np; i += 2) {
 		if (equation[i].kind == VARIABLE && equation[i].token.variable == IMAGINARY) {
 			if (*np + 2 > n_tokens) {
@@ -97,7 +107,7 @@ int		*np;
 					} while (combine_constants(equation, np, false));
 				} while (elim_k(equation, np));
 			} while (simp_pp(equation, np));
-		} while (fctor_power(equation, np));
+		} while (factor_power(equation, np));
 	} while (factor_times(equation, np));
 	simp_loop(equation, np);
 }
@@ -120,12 +130,13 @@ int		poly_flag, quick_flag;
 		simp_ssub(equation, np, (long) MATCH_ANY, 0.0, false, false, true, 1);
 	} while (uf_power(equation, np) || poly_factor(equation, np, false));
 	simp_ssub(equation, np, 0L, 0.0, true, false, true, 1);
-	side_debug(3, equation, *np);
 	rationalize(equation, np);
 	unsimp_power(equation, np, false);
 	uf_times(equation, np);
+	simp_loop(equation, np);
+	uf_repeat(equation, np);
 	do {
-		simp_loop(equation, np);
+		elim_loop(equation, np);
 	} while (mod_simp(equation, np));
 /* Here we try to simplify out unnecessary negative constants and imaginary numbers */
 	simp_i(equation, np);
@@ -153,7 +164,7 @@ int		poly_flag, quick_flag;
 	uf_times(equation, np);
 	do {
 		elim_loop(equation, np);
-	} while (fctor_power(equation, np));
+	} while (factor_power(equation, np));
 	uf_times(equation, np);
 	simp_loop(equation, np);
 	uf_power(equation, np);
@@ -164,9 +175,6 @@ int		poly_flag, quick_flag;
 	do {
 		simp_ssub(equation, np, 0L, 1.0, false, 2, true, 1);
 	} while (uf_power(equation, np));
-	make_fractions(equation, np);
-	uf_power(equation, np);
-	simp_loop(equation, np);
 	if (quick_flag) {
 do_quick:
 		uf_tsimp(equation, np);
@@ -185,6 +193,8 @@ do_quick:
 		}
 		uf_repeat(equation, np);
 		uf_tsimp(equation, np);
+		uf_repeat(equation, np);
+		uf_tsimp(equation, np);
 		blt(jmp_save, save_save, sizeof(jmp_save));
 	}
 	partial_flag = false;
@@ -213,10 +223,15 @@ try_again:
 		goto try_again;
 	}
 	partial_flag = true;
+	simp_constant_power(equation, np);
 	simp_ssub(equation, np, 0L, 1.0, true, false, true, 1);
 	unsimp_power(equation, np, false);
 	make_fractions(equation, np);
+	factor_power(equation, np);
 	uf_tsimp(equation, np);
+	make_fractions(equation, np);
+	uf_power(equation, np);
+	integer_root_simp(equation, np);
 	simpb_side(equation, np, 3);
 	while (div_remainder(equation, np, poly_flag, quick_flag)) {
 		simpb_side(equation, np, 3);
@@ -291,10 +306,9 @@ int		fc_level;
 		va[vc++].count = cnt;
 	}
 	if (vc == 0) {
-		simp_ssub(equation, np, 0L, 0.0, true, false, true, fc_level);
-		return;
+		goto skip_factor;
 	}
-	qsort((char *) &va[0], vc, sizeof(*va), vcmp);
+	qsort((char *) va, vc, sizeof(*va), vcmp);
 	simp2_divide(equation, np, va[0].v, fc_level);
 	for (i = 1; i < vc; i++) {
 		if (factor_divide(equation, np, va[i].v, 0.0))
@@ -309,6 +323,7 @@ int		fc_level;
 	while (factor_plus(equation, np, (long) MATCH_ANY, 0.0)) {
 		simp2_divide(equation, np, 0L, fc_level);
 	}
+skip_factor:
 	simp_ssub(equation, np, (long) MATCH_ANY, 0.0, true, false, true, fc_level);
 }
 
@@ -370,7 +385,7 @@ int		fc_level;
 				} while (factor_divide(equation, np, v, d));
 			} while (factor_plus(equation, np, v, d));
 		} while (super_code && super_factor(equation, np, super_code == 2));
-	} while (power_flag && fctor_power(equation, np));
+	} while (power_flag && factor_power(equation, np));
 }
 
 /*
@@ -465,8 +480,6 @@ organize(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	eloc;
-
 	if (*np <= 0 || (*np & 1) != 1) {
 		printf(_("Organize called with bad expression size!  n = %d\n"), *np);
 		longjmp(jmp_save, 13);
@@ -475,7 +488,135 @@ int		*np;
 		printf(_("Memory leak detected!  Expression larger than 'n_tokens'.\n"));
 		longjmp(jmp_save, 13);
 	}
-	org_recurse(equation, np, 0, 1, &eloc, 0);
+	org_recurse(equation, np, 0, 1, NULL, 0);
+}
+
+static int
+org_recurse(equation, np, loc, level, elocp, parent_op)
+token_type	*equation;
+int		*np, loc, level, *elocp;
+int		parent_op;
+{
+	token_type	*p1, *bp, *ep;
+	int	first_op, sub_op;
+	int	op_count, op_loc;
+	int	neg_one_times;
+	int	i;
+	int	eloc;
+	int	sub_eloc;
+	int	min1;
+	int	invert;
+
+	bp = &equation[loc];
+	ep = &equation[*np];
+	min1 = bp->level;
+	for (p1 = bp + 1; p1 < ep; p1 += 2) {
+		if (p1->level < min1) {
+			if (p1->level < level)
+				break;
+			min1 = p1->level;
+		}
+	}
+	ep = p1;
+	eloc = (ep - equation) - 1;
+	if (elocp)
+		*elocp = eloc;
+	if (eloc == loc) {
+		bp->level = max(level - 1, 1);
+		return 0;
+	}
+	if (min1 > level) {
+		for (p1 = bp; p1 < ep; p1++)
+			p1->level -= min1 - level;
+	}
+	op_count = 0;
+	first_op = 0;
+	for (p1 = bp + 1; p1 < ep; p1 += 2) {
+		if (p1->level == level) {
+			op_count++;
+			if (first_op == 0) {
+				first_op = p1->token.operatr;
+				op_loc = p1 - equation;
+			} else
+				break;
+		}
+	}
+	neg_one_times = (op_count == 1
+	    && (parent_op == PLUS || parent_op == MINUS)
+	    && ((op_loc - loc == 1
+	    && first_op == TIMES
+	    && bp->kind == CONSTANT
+	    && bp->token.constant == -1.0)
+	    || (eloc - op_loc == 1
+	    && (first_op == TIMES || first_op == DIVIDE)
+	    && (ep - 1)->kind == CONSTANT
+	    && (ep - 1)->token.constant == -1.0)));
+	if (neg_one_times && (op_loc - loc != 1)) {
+		blt(bp + 2, bp, (eloc - loc - 1) * sizeof(*equation));
+		bp->level = level;
+		bp->kind = CONSTANT;
+		bp->token.constant = -1.0;
+		(bp + 1)->level = level;
+		(bp + 1)->kind = OPERATOR;
+		(bp + 1)->token.operatr = TIMES;
+	}
+	for (i = loc; i <= eloc; i += 2) {
+		if (equation[i].level > level) {
+			sub_op = org_recurse(equation, np, i, level + 1, &sub_eloc, first_op);
+			switch (sub_op) {
+			case PLUS:
+			case MINUS:
+				if (!neg_one_times && first_op != PLUS && first_op != MINUS)
+					break;
+				if (i - 1 >= loc && equation[i-1].token.operatr == MINUS)
+					invert = true;
+				else
+					invert = neg_one_times;
+				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
+				if (neg_one_times) {
+					binary_parenthesize(equation, sub_eloc + 1, loc + 1);
+					org_recurse(equation, np, loc, level + 1, NULL, PLUS);
+					first_op = sub_op;
+				}
+				break;
+			case TIMES:
+			case DIVIDE:
+				if (first_op != TIMES && first_op != DIVIDE)
+					break;
+				invert = (i - 1 >= loc && equation[i-1].token.operatr == DIVIDE);
+				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
+				break;
+			}
+			i = sub_eloc;
+		}
+	}
+	return first_op;
+}
+
+static void
+org_up_level(bp, ep, level, invert)
+token_type	*bp, *ep;
+int		level, invert;
+{
+	for (; bp <= ep; bp++) {
+		bp->level--;
+		if (invert && bp->level == level && bp->kind == OPERATOR) {
+			switch (bp->token.operatr) {
+			case PLUS:
+				bp->token.operatr = MINUS;
+				break;
+			case MINUS:
+				bp->token.operatr = PLUS;
+				break;
+			case TIMES:
+				bp->token.operatr = DIVIDE;
+				break;
+			case DIVIDE:
+				bp->token.operatr = TIMES;
+				break;
+			}
+		}
+	}
 }
 
 /*
@@ -509,14 +650,16 @@ int		*np;
 				if (equation[j].level <= level) {
 					if (left_level <= equation[j].level
 					    && equation[j].token.operatr == POWER) {
-						if (!symb_flag && (i + 2) == j && equation[j].level == equation[j+1].level
-						    && equation[j-1].kind == CONSTANT
-						    && equation[j+1].kind == CONSTANT) {
+						if (!symb_flag
+						    && (i + 2) == j && equation[j-1].kind == CONSTANT) {
 							f_to_fraction(equation[j-1].token.constant, &numerator, &denominator);
 							if (fmod(numerator, 2.0) == 0.0) {
-								f_to_fraction(equation[j+1].token.constant, &numerator2, &denominator2);
-								if (fmod(denominator2, 2.0) == 0.0) {
-									break;
+								if (equation[j].level == equation[j+1].level
+								    && equation[j+1].kind == CONSTANT) {
+									f_to_fraction(equation[j+1].token.constant, &numerator2, &denominator2);
+									if (fmod(denominator2, 2.0) == 0.0) {
+										break;
+									}
 								}
 							}
 						}
@@ -540,9 +683,142 @@ int		*np;
 	return modified;
 }
 
+/*
+ * Simplify things like 12^(1/2) to 2*3^(1/2).
+ * Return true if equation side was modified.
+ */
+int
+integer_root_simp(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	modified;
+	int	i, j, k;
+	int	level;
+	double	d1;
+	int	root;
+
+	modified = false;
+	for (i = 1; i < (*np - 2); i += 2) {
+		if (equation[i].token.operatr == POWER) {
+			level = equation[i].level;
+			if (equation[i-1].level == level
+			    && equation[i+2].level == level + 1
+			    && equation[i+2].token.operatr == DIVIDE
+			    && equation[i+3].level == level + 1
+			    && equation[i-1].kind == CONSTANT
+			    && equation[i+1].kind == CONSTANT
+			    && equation[i+1].token.constant == 1.0
+			    && equation[i+3].kind == CONSTANT) {
+				if (i + 4 < *np && equation[i+4].level >= level)
+					continue;
+				if (equation[i+3].token.constant > 50.0
+				    || equation[i+3].token.constant < 2.0)
+					continue;
+				root = equation[i+3].token.constant;
+				if (root != equation[i+3].token.constant
+				    || root < 2)
+					continue;
+				if (!factor_one(equation[i-1].token.constant))
+					continue;
+				d1 = 1.0;
+				for (j = 0; j < uno; j++) {
+					while (ucnt[j] >= root) {
+						d1 *= unique[j];
+						ucnt[j] -= root;
+					}
+				}
+				if (d1 == 1.0)
+					continue;
+				if (*np + 2 > n_tokens) {
+					error_huge();
+				}
+				equation[i-1].token.constant = multiply_out_unique();
+				k = i - 1;
+				j = i + 4;
+				for (; k < j; k++) {
+					equation[k].level++;
+				}
+				blt(&equation[i+1], &equation[i-1], (*np - (i - 1)) * sizeof(token_type));
+				*np += 2;
+				equation[i].level = level;
+				equation[i].token.operatr = TIMES;
+				equation[i-1].level = level;
+				equation[i-1].kind = CONSTANT;
+				equation[i-1].token.constant = d1;
+				modified = true;
+				i += 2;
+			}
+		}
+	}
+	return modified;
+}
+
+/*
+ * Simplify constant^(constant*x).
+ * Return true if equation side was modified.
+ */
+int
+simp_constant_power(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	modified;
+	int	i;
+
+	modified = false;
+	for (i = 1; i < *np; i += 2) {
+		if (equation[i].token.operatr == POWER) {
+			modified |= sp2_sub(equation, np, i);
+		}
+	}
+	return modified;
+}
+
+static int
+sp2_sub(equation, np, i)
+token_type	*equation;
+int		*np, i;
+{
+	int	j;
+	int	level;
+
+	level = equation[i].level;
+	if (equation[i-1].level != level || equation[i-1].kind != CONSTANT)
+		return false;
+	if (equation[i+1].level != level + 1 || equation[i+1].kind != CONSTANT
+	    || equation[i+1].token.constant == 1.0)
+		return false;
+	if (equation[i-1].token.constant < 0.0 && fmod(equation[i+1].token.constant, 1.0) != 0.0)
+		return false;	/* don't make imaginary numbers */
+	j = i + 2;
+	if (j >= *np || equation[j].level != level + 1)
+		return false;
+	switch (equation[j].token.operatr) {
+	case TIMES:
+		break;
+	case DIVIDE:
+		if (*np + 2 > n_tokens) {
+			error_huge();
+		}
+		blt(&equation[j+2], &equation[j], (*np - j) * sizeof(*equation));
+		*np += 2;
+		equation[j+1].level = level + 1;
+		equation[j+1].kind = CONSTANT;
+		equation[j+1].token.constant = 1.0;
+		break;
+	default:
+		return false;
+	}
+	equation[j].level = level;
+	equation[j].token.operatr = POWER;
+	equation[i-1].level++;
+	equation[i].level++;
+	return true;
+}
+
 /*
  * Convert x^-y to 1/(x^y).
- * Simplify k1^(k2*x).
  * Return true if equation side was modified.
  */
 int
@@ -559,17 +835,10 @@ int		*np;
 			modified |= sp_sub(equation, np, i);
 		}
 	}
-	if (modified)
-		return true;
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			modified |= sp2_sub(equation, np, i);
-		}
-	}
 	return modified;
 }
 
-int
+static int
 sp_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
@@ -627,176 +896,6 @@ int		*np, i;
 	return false;
 }
 
-int
-sp2_sub(equation, np, i)
-token_type	*equation;
-int		*np, i;
-{
-	int	j;
-	int	level;
-
-	level = equation[i].level;
-	if (equation[i-1].level != level || equation[i-1].kind != CONSTANT)
-		return false;
-	if (equation[i+1].level != level + 1 || equation[i+1].kind != CONSTANT
-	    || equation[i+1].token.constant == 1.0)
-		return false;
-	j = i + 2;
-	if (j >= *np || equation[j].level != level + 1)
-		return false;
-	switch (equation[j].token.operatr) {
-	case TIMES:
-		break;
-	case DIVIDE:
-		if (*np + 2 > n_tokens) {
-			error_huge();
-		}
-		blt(&equation[j+2], &equation[j], (*np - j) * sizeof(*equation));
-		*np += 2;
-		equation[j+1].level = level + 1;
-		equation[j+1].kind = CONSTANT;
-		equation[j+1].token.constant = 1.0;
-		break;
-	default:
-		return false;
-	}
-	equation[j].level = level;
-	equation[j].token.operatr = POWER;
-	equation[i-1].level++;
-	equation[i].level++;
-	return true;
-}
-
-int
-org_recurse(equation, np, loc, level, elocp, parent_op)
-token_type	*equation;
-int		*np, loc, level, *elocp;
-int		parent_op;
-{
-	token_type	*p1, *bp, *ep;
-	int	first_op, sub_op;
-	int	op_count, op_loc;
-	int	neg_one_times;
-	int	i;
-	int	junk;
-	int	eloc;
-	int	sub_eloc;
-	int	min1;
-	int	invert;
-
-	bp = &equation[loc];
-	ep = &equation[*np];
-	min1 = bp->level;
-	for (p1 = bp + 1; p1 < ep; p1 += 2) {
-		if (p1->level < min1) {
-			if (p1->level < level)
-				break;
-			min1 = p1->level;
-		}
-	}
-	ep = p1;
-	eloc = (ep - equation) - 1;
-	*elocp = eloc;
-	if (eloc == loc) {
-		bp->level = max(level - 1, 1);
-		return 0;
-	}
-	if (min1 > level) {
-		for (p1 = bp; p1 < ep; p1++)
-			p1->level -= min1 - level;
-	}
-	op_count = 0;
-	first_op = 0;
-	for (p1 = bp + 1; p1 < ep; p1 += 2) {
-		if (p1->level == level) {
-			op_count++;
-			if (first_op == 0) {
-				first_op = p1->token.operatr;
-				op_loc = p1 - equation;
-			} else
-				break;
-		}
-	}
-	neg_one_times = (op_count == 1
-	    && (parent_op == PLUS || parent_op == MINUS)
-	    && ((op_loc - loc == 1
-	    && first_op == TIMES
-	    && bp->kind == CONSTANT
-	    && bp->token.constant == -1.0)
-	    || (eloc - op_loc == 1
-	    && (first_op == TIMES || first_op == DIVIDE)
-	    && (ep - 1)->kind == CONSTANT
-	    && (ep - 1)->token.constant == -1.0)));
-	if (neg_one_times && (op_loc - loc != 1)) {
-		blt(bp + 2, bp, (eloc - loc - 1) * sizeof(*equation));
-		bp->level = level;
-		bp->kind = CONSTANT;
-		bp->token.constant = -1.0;
-		(bp + 1)->level = level;
-		(bp + 1)->kind = OPERATOR;
-		(bp + 1)->token.operatr = TIMES;
-	}
-	for (i = loc; i <= eloc; i += 2) {
-		if (equation[i].level > level) {
-			sub_op = org_recurse(equation, np, i, level + 1, &sub_eloc, first_op);
-			switch (sub_op) {
-			case PLUS:
-			case MINUS:
-				if (!neg_one_times && first_op != PLUS && first_op != MINUS)
-					break;
-				if (i - 1 >= loc && equation[i-1].token.operatr == MINUS)
-					invert = true;
-				else
-					invert = neg_one_times;
-				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
-				if (neg_one_times) {
-					binary_parenthesize(equation, sub_eloc + 1, loc + 1);
-					org_recurse(equation, np, loc, level + 1, &junk, PLUS);
-					first_op = sub_op;
-				}
-				break;
-			case TIMES:
-			case DIVIDE:
-				if (first_op != TIMES && first_op != DIVIDE)
-					break;
-				if (i - 1 >= loc && equation[i-1].token.operatr == DIVIDE)
-					invert = true;
-				else
-					invert = false;
-				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
-				break;
-			}
-			i = sub_eloc;
-		}
-	}
-	return first_op;
-}
-
-org_up_level(bp, ep, level, invert)
-token_type	*bp, *ep;
-int		level, invert;
-{
-	for (; bp <= ep; bp++) {
-		bp->level--;
-		if (invert && bp->level == level && bp->kind == OPERATOR) {
-			switch (bp->token.operatr) {
-			case PLUS:
-				bp->token.operatr = MINUS;
-				break;
-			case MINUS:
-				bp->token.operatr = PLUS;
-				break;
-			case TIMES:
-				bp->token.operatr = DIVIDE;
-				break;
-			case DIVIDE:
-				bp->token.operatr = TIMES;
-				break;
-			}
-		}
-	}
-}
-
 /*
  * Combine two or more constants on the same level.
  * If "iflag" is false, don't produce imaginary numbers.
@@ -812,7 +911,7 @@ int		iflag;
 	return const_recurse(equation, np, 0, 1, iflag);
 }
 
-int
+static int
 const_recurse(equation, np, loc, level, iflag)
 token_type	*equation;
 int		*np, loc, level, iflag;
@@ -825,8 +924,7 @@ int		*np, loc, level, iflag;
 	double		d;
 	double		d1, d2, numerator, denominator;
 	int		op;
-	int		fflag;
-	complexs	cv;
+	complexs	cv, p;
 
 	modified = false;
 	zero = 0;
@@ -857,13 +955,14 @@ cheat:
 				    &d, op, d2)) {
 					if (op == POWER
 					    && !domain_check) {
-						fflag = f_to_fraction(d2, &numerator, &denominator);
-						if (!fflag) {
-							if (!iflag)
+						if (!f_to_fraction(d2, &numerator, &denominator)) {	/* if irrational power */
+							if (!iflag || (preserve_roots && !in_calc_cmd))
 								return modified;
 							cv.re = d;
 							cv.im = 0.0;
-							cv = cpow(cv, d2);
+							p.re = d2;
+							p.im = 0.0;
+							cv = complex_pow(cv, p);
 							if (*np + 2 > n_tokens) {
 								error_huge();
 							}
@@ -1025,6 +1124,9 @@ double	k2;
 		break;
 	case MODULUS:
 		*k1p = modf(*k1p / k2, &d) * k2;
+		if (true_modulus && *k1p < 0.0) {
+			*k1p += fabs(k2);	/* make positive */
+		}
 		break;
 	case POWER:
 		domain_check = true;
@@ -1032,6 +1134,12 @@ double	k2;
 			domain_check = false;
 			break;
 		}
+		if (*k1p == 0.0 && k2 < 0.0) {
+#if	!SILENT
+			printf(_("Divide by zero error!\n"));
+#endif
+			longjmp(jmp_save, 2);
+		}
 		if (*k1p == 0.0 && k2 == 0.0) {
 #if	!SILENT
 			printf(_("Warning: 0 raised to the power of 0.\n"));
@@ -1039,10 +1147,9 @@ double	k2;
 			d = 1.0;	/* some people don't know 0^0 */
 		} else {
 			d = pow(*k1p, k2);
-			if (preserve_roots) {
-				if (fmod(k2, 1.0) != 0.0
-				    && (fmod(*k1p, 1.0) == 0.0 || f_to_fraction(*k1p, &d1, &d2))) {
-					if (fmod(d, 1.0) != 0.0 && !f_to_fraction(d, &d1, &d2)) {
+			if (preserve_roots && !in_calc_cmd) {
+				if (fmod(k2, 1.0) != 0.0 && f_to_fraction(*k1p, &d1, &d2)) {
+					if (!f_to_fraction(d, &d1, &d2)) {
 						domain_check = false;
 						return false;
 					}
@@ -1061,6 +1168,18 @@ double	k2;
 		d = exp(lgamma(*k1p + 1.0));
 		check_err();
 #else
+		if (*k1p < 0.0) {
+#if	!SILENT
+			printf(_("Factorial not defined for negative numbers!\n"));
+#endif
+			longjmp(jmp_save, 2);
+		}
+		if (fmod(*k1p, 1.0) != 0.0) {
+#if	!SILENT
+			printf(_("Factorial operand not integer!\n"));
+#endif
+			longjmp(jmp_save, 2);
+		}
 		d = 1.0;
 		for (d1 = 2.0; d1 <= *k1p; d1 += 1.0) {
 			d *= d1;
@@ -1075,8 +1194,8 @@ double	k2;
 }
 
 /*
- * Eliminate operations that involve 0 or 1 that can be simplified.
- * Fix addition/subtraction of negative constants and variables.
+ * Eliminate operations that involve constants that can be simplified.
+ * Fix addition/subtraction of negative constants and expressions.
  * Replace division by a constant with times its reciprocal.
  * Return true if one or more eliminations occurred.
  */
@@ -1085,12 +1204,13 @@ elim_k(equation, np)
 token_type	*equation;
 int		*np;
 {
-	token_type	*p1, *p2;
+	token_type	*p1, *p2, *p3, *p4;
 	token_type	*ep;
 	int		modified;
 	int		level;
 	double		d;
 	double		numerator, denominator;
+	int		flag;
 
 	modified = false;
 	for (p1 = equation + 1;;) {
@@ -1102,31 +1222,68 @@ int		*np;
 			switch (p1->token.operatr) {
 			case PLUS:
 			case MINUS:
+				p2 = p1 + 1;
 				if (p1 + 2 < ep && (p1 + 2)->level == level + 1
 				    && ((p1 + 2)->token.operatr == TIMES || (p1 + 2)->token.operatr == DIVIDE)
-				    && (p1 + 1)->kind == CONSTANT
-				    && (p1 + 1)->token.constant < 0.0) {
+				    && p2->kind == CONSTANT
+				    && p2->token.constant < 0.0) {
 					if (p1->token.operatr == PLUS)
 						p1->token.operatr = MINUS;
 					else
 						p1->token.operatr = PLUS;
-					(p1 + 1)->token.constant = -((p1 + 1)->token.constant);
+					p2->token.constant = -(p2->token.constant);
 				}
-				if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
-					if ((p1 + 1)->token.constant < 0.0) {
+				if (p2->level == level && p2->kind == CONSTANT) {
+					if (p2->token.constant < 0.0) {
 						if (p1->token.operatr == PLUS)
 							p1->token.operatr = MINUS;
 						else
 							p1->token.operatr = PLUS;
-						(p1 + 1)->token.constant = -(p1 + 1)->token.constant;
+						p2->token.constant = -p2->token.constant;
 					}
-					if ((p1 + 1)->token.constant == 0.0) {
+					if (p2->token.constant == 0.0) {
 						blt(p1, p1 + 2, (int) ((char *) ep - (char *) (p1 + 2)));
 						*np -= 2;
 						modified = true;
 						continue;
 					}
 				}
+				if ((p1 - 1)->level == level && (p1 - 1)->kind == CONSTANT
+				    && isinf((p1 - 1)->token.constant)) {
+					p2 = p1 - 1;
+				}
+				if (p2->level == level && p2->kind == CONSTANT && isinf(p2->token.constant)) {
+					flag = false;
+					for (p3 = p1;; p3--) {
+						if (p3->level < level) {
+							p3++;
+							break;
+						}
+						if (p3->kind == CONSTANT && p3 != p2
+						    && !isfinite(p3->token.constant)) {
+							flag = true;
+						}
+						if (p3 == equation)
+							break;
+					}
+					for (p4 = p1; p4 < ep && p4->level >= level; p4++) {
+						if (p4->kind == CONSTANT && p4 != p2
+						    && !isfinite(p4->token.constant)) {
+							flag = true;
+						}
+					}
+					if (!flag) { /* no other infinities on level */
+						if (p2 > p3 && (p2 - 1)->token.operatr == MINUS) {
+							p2->token.constant = -(p2->token.constant);
+						}
+						blt(p2 + 1, p4, (int) ((char *) ep - (char *) p4));
+						*np -= p4 - (p2 + 1);
+						ep = &equation[*np];
+						blt(p3, p2, (int) ((char *) ep - (char *) p2));
+						*np -= p2 - p3;
+						return true;
+					}
+				}
 				break;
 			}
 			switch (p1->token.operatr) {
@@ -1156,7 +1313,8 @@ int		*np;
 				break;
 			case TIMES:
 				if ((p1 - 1)->level == level && (p1 - 1)->kind == CONSTANT) {
-					if ((p1 - 1)->token.constant == 0.0) {
+					d = (p1 - 1)->token.constant;
+					if (d == 0.0) {
 						for (p2 = p1 + 2; p2 < ep; p2 += 2) {
 							if (p2->level < level)
 								break;
@@ -1167,9 +1325,9 @@ int		*np;
 						continue;
 					}
 #if	EPSILON
-					if (fabs((p1 - 1)->token.constant - 1.0) <= epsilon) {
+					if (fabs(d - 1.0) <= epsilon) {
 #else
-					if ((p1 - 1)->token.constant == 1.0) {
+					if (d == 1.0) {
 #endif
 						blt(p1 - 1, p1 + 1, (int) ((char *) ep - (char *) (p1 + 1)));
 						*np -= 2;
@@ -1179,28 +1337,7 @@ int		*np;
 				}
 				if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
 					d = (p1 + 1)->token.constant;
-					if (d == 0.0) {
-						for (p2 = p1 - 1; p2 > &equation[0]; p2--) {
-							if ((p2 - 1)->level < level)	/* No mistake.  Wipe out level! */
-								break;
-						}
-						blt(p2, p1 + 1, (int) ((char *) ep - (char *) (p1 + 1)));
-						*np -= (p1 + 1) - p2;
-						p1 = p2 + 1;
-						modified = true;
-						continue;
-					}
-#if	EPSILON
-					if (fabs(d - 1.0) <= epsilon) {
-#else
-					if (d == 1.0) {
-#endif
-						blt(p1, p1 + 2, (int) ((char *) ep - (char *) (p1 + 2)));
-						*np -= 2;
-						modified = true;
-						continue;
-					}
-					for (p2 = p1 - 1; p2 > &equation[0]; p2--) {
+					for (p2 = p1 - 1; p2 > equation; p2--) {
 						if ((p2 - 1)->level < level)
 							break;
 					}
@@ -1214,9 +1351,12 @@ int		*np;
 					(p2 + 1)->level = level;
 					(p2 + 1)->kind = OPERATOR;
 					(p2 + 1)->token.operatr = TIMES;
-					if (p2 > &equation[0]) {
+					if (p2 > equation) {
 						p1 = p2 - 1;
 						continue;
+					} else {
+						p1 = equation + 1;
+						continue;
 					}
 				}
 				break;
@@ -1235,15 +1375,8 @@ int		*np;
 				}
 				p2 = p1 + 1;
 				if (p2->level == level && p2->kind == CONSTANT) {
-#if	true
-					if (f_to_fraction(p2->token.constant, &numerator, &denominator)) {
-						p2->token.constant = denominator / numerator;
-					} else {
-						p2->token.constant = 1.0 / p2->token.constant;
-					}
-#else
-					p2->token.constant = 1.0 / p2->token.constant;
-#endif
+					f_to_fraction(p2->token.constant, &numerator, &denominator);
+					p2->token.constant = denominator / numerator;
 					p1->token.operatr = TIMES;
 					continue;
 				}
@@ -1254,6 +1387,21 @@ int		*np;
 				}
 				break;
 			case MODULUS:
+				p2 = p1 + 1;
+				if (p2->level == level && p2->kind == CONSTANT) {
+					if (p2->token.constant == 0.0) {
+#if	!SILENT
+						printf(_("Divide by zero error!\n"));
+#endif
+						longjmp(jmp_save, 2);
+					}
+					if (isinf(p2->token.constant)) {
+						blt(p1, p1 + 2, (int) ((char *) ep - (char *) (p1 + 2)));
+						*np -= 2;
+						modified = true;
+						continue;
+					}
+				}
 				p2 = p1 - 1;
 				if (p2->level == level && p2->kind == CONSTANT
 				    && p2->token.constant == 0.0) {
@@ -1282,7 +1430,7 @@ int		*np;
 				}
 				if ((p1 + 1)->level == level && (p1 + 1)->kind == CONSTANT) {
 					if ((p1 + 1)->token.constant == 0.0) {
-						for (p2 = p1 - 1; p2 > &equation[0]; p2--) {
+						for (p2 = p1 - 1; p2 > equation; p2--) {
 							if ((p2 - 1)->level <= level)
 								break;
 						}
@@ -1335,7 +1483,7 @@ int		*diff_signp;
 	return compare_recurse(p1, n1, l1, p2, n2, l2, diff_signp);
 }
 
-int
+static int
 compare_recurse(p1, n1, l1, p2, n2, l2, diff_signp)
 token_type	*p1;
 int		n1, l1;
@@ -1354,10 +1502,12 @@ int		n2, l2, *diff_signp;
 	double		d1, c1, c2;
 
 	*diff_signp = false;
+#if	!SILENT
 	if ((n1 & 1) != 1 || (n2 & 1) != 1) {
 		printf(_("Programming error in call to compare_recurse()!\n"));
 		longjmp(jmp_save, 13);
 	}
+#endif
 	if (n1 == 1 && n2 == 1) {
 		if (p1->kind != p2->kind)
 			return false;
@@ -1782,7 +1932,7 @@ int		*np;
 			if ((eloc - (i + 1)) + 5 + (eiloc - biloc) + *np + 2 > n_tokens)
 				error_huge();
 			n = eloc - (i + 1);
-			blt(&scratch[0], &equation[i+1], n * sizeof(token_type));
+			blt(scratch, &equation[i+1], n * sizeof(token_type));
 			scratch[iloc-(i+1)].kind = CONSTANT;
 			scratch[iloc-(i+1)].token.constant = 0.0;
 			for (j = 0; j < n; j++)
@@ -1827,7 +1977,7 @@ int		*np;
 			equation[iloc].level = ilevel + 1;
 			blt(&equation[i+1+n], &equation[i], (*np - i) * sizeof(token_type));
 			*np += n + 1;
-			blt(&equation[i+1], &scratch[0], n * sizeof(token_type));
+			blt(&equation[i+1], scratch, n * sizeof(token_type));
 			equation[i+1+n].token.operatr = TIMES;
 			return true;
 		}
@@ -1850,7 +2000,7 @@ int		*np;
 	return(order_recurse(equation, np, 0, 1));
 }
 
-int
+static int
 order_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
@@ -1942,9 +2092,9 @@ int		*np, loc, level;
 					if (equation[n].level <= level)
 						break;
 				n -= k;
-				blt(&scratch[0], &equation[k], n * sizeof(*equation));
+				blt(scratch, &equation[k], n * sizeof(*equation));
 				blt(&equation[j+n], &equation[j], (k - j) * sizeof(*equation));
-				blt(&equation[j], &scratch[0], n * sizeof(*equation));
+				blt(&equation[j], scratch, n * sizeof(*equation));
 				j += n;
 				k += n;
 				modified = true;
@@ -1957,6 +2107,7 @@ int		*np, loc, level;
 	return modified;
 }
 
+static void
 swap(equation, np, level, i1, i2)
 token_type	*equation;
 int		*np;
@@ -1976,10 +2127,10 @@ int		i1, i2;
 	}
 	n1 = e1 - i1;
 	n2 = e2 - i2;
-	blt(&scratch[0], &equation[i1], (e2 - i1) * sizeof(*equation));
+	blt(scratch, &equation[i1], (e2 - i1) * sizeof(*equation));
 	blt(&equation[i1+n2], &equation[e1], (i2 - e1) * sizeof(*equation));
 	blt(&equation[i1], &scratch[i2-i1], n2 * sizeof(*equation));
-	blt(&equation[e2-n1], &scratch[0], n1 * sizeof(*equation));
+	blt(&equation[e2-n1], scratch, n1 * sizeof(*equation));
 }
 
 /*
@@ -2017,10 +2168,6 @@ be_thorough:
 					count++;
 				}
 			}
-/*
-			if (count != 1)
-				continue;
-*/
 			flag = false;
 			for (j = i + 2; j < end_loc; j += 2) {
 				if (equation[j].level == (div_level + 1)) {
@@ -2051,18 +2198,18 @@ do_again:
 						flag = 2;
 						break;
 					}
-					if (flag)
-						break;
+					break;
 				}
 			}
 			if (flag) {
 				for (k = j - 2; k > i1; k -= 2) {
-					if (equation[k].level == (div_level + 2)
-					    || (equation[k].level == (div_level + 3) && flag == 1)) {
-						if (equation[k].token.operatr == POWER
-						    && equation[k].level == equation[k+1].level
-						    && equation[k+1].kind == CONSTANT
-						    && fmod(equation[k+1].token.constant, 1.0) == 0.5) {
+					if ((equation[k].level == (div_level + 2)
+					    || (equation[k].level == (div_level + 3) && flag == 1))
+					    && equation[k].token.operatr == POWER
+					    && equation[k].level == equation[k+1].level
+					    && equation[k+1].kind == CONSTANT
+					    && fmod(equation[k+1].token.constant, 1.0) == 0.5) {
+						if (count != 1) {
 							for (k1 = i + 2; k1 < end_loc; k1 += 2) {
 								if (equation[k1].token.operatr == POWER
 								    && equation[k1].level == equation[k1+1].level
@@ -2073,48 +2220,49 @@ do_again:
 									goto be_thorough;
 								}
 							}
-							neg_one_loc = i1 + 1;
-							k = i1 - i;
-							blt(scratch, &equation[i+1], k * sizeof(token_type));
-							scratch[k].level = div_level + 2;
-							scratch[k].kind = CONSTANT;
-							scratch[k].token.constant = -1.0;
-							k++;
-							scratch[k].level = div_level + 2;
-							scratch[k].kind = OPERATOR;
-							scratch[k].token.operatr = TIMES;
-							k++;
-							blt(&scratch[k], &equation[neg_one_loc], (end_loc - neg_one_loc) * sizeof(token_type));
-							for (k1 = 0; k1 < (j - neg_one_loc); k1++, k++)
-								scratch[k].level++;
-							k = end_loc - (i + 1) + 2;
-							if (*np + 2 * (k + 1) > n_tokens) {
-								error_huge();
-							}
-							blt(&equation[end_loc+(2*(k+1))], &equation[end_loc], (*np - end_loc) * sizeof(token_type));
-							*np += 2 * (k + 1);
-							k1 = end_loc;
-							equation[k1].level = div_level;
-							equation[k1].kind = OPERATOR;
-							equation[k1].token.operatr = TIMES;
-							k1++;
-							blt(&equation[k1], scratch, k * sizeof(token_type));
-							k1 += k;
-							equation[k1].level = div_level;
-							equation[k1].kind = OPERATOR;
-							equation[k1].token.operatr = DIVIDE;
-							k1++;
-							blt(&equation[k1], scratch, k * sizeof(token_type));
-							modified = true;
-							i = k1 + k;
-							goto be_thorough;
 						}
+						neg_one_loc = i1 + 1;
+						k = i1 - i;
+						blt(scratch, &equation[i+1], k * sizeof(token_type));
+						scratch[k].level = div_level + 2;
+						scratch[k].kind = CONSTANT;
+						scratch[k].token.constant = -1.0;
+						k++;
+						scratch[k].level = div_level + 2;
+						scratch[k].kind = OPERATOR;
+						scratch[k].token.operatr = TIMES;
+						k++;
+						blt(&scratch[k], &equation[neg_one_loc], (end_loc - neg_one_loc) * sizeof(token_type));
+						for (k1 = 0; k1 < (j - neg_one_loc); k1++, k++)
+							scratch[k].level++;
+						k = end_loc - (i + 1) + 2;
+						if (*np + 2 * (k + 1) > n_tokens) {
+							error_huge();
+						}
+						blt(&equation[end_loc+(2*(k+1))], &equation[end_loc], (*np - end_loc) * sizeof(token_type));
+						*np += 2 * (k + 1);
+						k1 = end_loc;
+						equation[k1].level = div_level;
+						equation[k1].kind = OPERATOR;
+						equation[k1].token.operatr = TIMES;
+						k1++;
+						blt(&equation[k1], scratch, k * sizeof(token_type));
+						k1 += k;
+						equation[k1].level = div_level;
+						equation[k1].kind = OPERATOR;
+						equation[k1].token.operatr = DIVIDE;
+						k1++;
+						blt(&equation[k1], scratch, k * sizeof(token_type));
+						modified = true;
+						i = k1 + k;
+						goto be_thorough;
 					}
 				}
 			}
 			if (j >= end_loc)
 				continue;
-			for (i1 = j, j += 2; j < end_loc; j += 2) {
+			i1 = j;
+			for (j += 2; j < end_loc; j += 2) {
 				if (equation[j].level == (div_level + 1)) {
 					break;
 				}
