@@ -7,15 +7,9 @@
 #include "am.h"
 #include "externs.h"
 
-/*
- * This is the subroutine used by the "unfactor" command.
- */
-unfa_sub(n)
-int	n;
-{
-	uf_simp(&lhs[n][0], &n_lhs[n]);
-	uf_simp(&rhs[n][0], &n_rhs[n]);
-}
+static int	unf_sub();
+static int	usp2_sub();
+static void	no_divide();
 
 /*
  * Unfactor equation side and simplify.
@@ -75,8 +69,7 @@ int		*np;
 {
 	int	rv;
 
-	if (debug_level >= 5)
-		printf("uf_power()\n");
+	debug_string(5, "uf_power()");
 	organize(equation, np);
 	rv = sub_ufactor(equation, np, 2);
 	if (rv) {
@@ -97,8 +90,7 @@ int		*np;
 {
 	int	rv;
 
-	if (debug_level >= 5)
-		printf("uf_pplus()\n");
+	debug_string(5, "uf_pplus()");
 	organize(equation, np);
 	rv = sub_ufactor(equation, np, 4);
 	if (rv) {
@@ -122,24 +114,33 @@ int		*np;
 
 /*
  * Unfactor power only.
- * a^2 -> a * a
- * Return true if equation side is modified.
+ * (a + 1)^2 -> (a + 1)*(a + 1)
  */
-int
 uf_repeat(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	rv;
-
-	if (debug_level >= 5)
-		printf("uf_repeat()\n");
+	debug_string(5, "uf_repeat()");
 	organize(equation, np);
-	rv = sub_ufactor(equation, np, 6);
-	if (rv) {
+	if (sub_ufactor(equation, np, 6)) {
+		organize(equation, np);
+	}
+	patch_root_div(equation, np);
+}
+
+/*
+ * Unfactor power only.
+ * a^2 -> a*a
+ * Useful for removing all integer powers.
+ */
+uf_repeat_always(equation, np)
+token_type	*equation;
+int		*np;
+{
+	organize(equation, np);
+	if (sub_ufactor(equation, np, 8)) {
 		organize(equation, np);
 	}
-	return rv;
 }
 
 /*
@@ -157,8 +158,7 @@ int		*np;
 	int	rv;
 
 	rv = false;
-	if (debug_level >= 5)
-		printf("uf_times()\n");
+	debug_string(5, "uf_times()");
 	do {
 		organize(equation, np);
 		if (partial_flag)
@@ -173,6 +173,7 @@ int		*np;
 	return rv;
 }
 
+static void
 no_divide(equation, np)
 token_type	*equation;
 int		*np;
@@ -334,7 +335,7 @@ end_uf:
 }
 #endif
 
-int
+static int
 unf_sub(equation, np, b1, loc, e1, level, ii)
 token_type	*equation;
 int		*np;
@@ -405,7 +406,7 @@ u_again:
 					}
 					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(*equation));
 					*np += len - (e1 - b1);
-					blt(&equation[b1], &scratch[0], len * sizeof(*equation));
+					blt(&equation[b1], scratch, len * sizeof(*equation));
 					return true;
 				}
 			}
@@ -452,7 +453,7 @@ u1_again:
 					}
 					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(*equation));
 					*np += len - (e1 - b1);
-					blt(&equation[b1], &scratch[0], len * sizeof(*equation));
+					blt(&equation[b1], scratch, len * sizeof(*equation));
 					return true;
 				}
 			}
@@ -502,41 +503,30 @@ u2_again:
 					}
 					blt(&equation[b1+len], &equation[e1], (*np - e1) * sizeof(*equation));
 					*np += len - (e1 - b1);
-					blt(&equation[b1], &scratch[0], len * sizeof(*equation));
+					blt(&equation[b1], scratch, len * sizeof(*equation));
 					return true;
 				}
 			}
 		}
 do_repeat:
-		if (ii != 6)
+		if (ii != 6 && ii != 8)
 			break;
 		i = loc;
 		if (equation[i+1].level != level
 		    || equation[i+1].kind != CONSTANT
 		    || equation[i+1].token.constant <= 1.0)
 			break;
-#if	true	/* set this because polynomial factoring is not perfect */
-		if (equation[i+1].token.constant > 2.0 && fmod(equation[i+1].token.constant, 1.0) != 0.0)
-			break;
-#endif
-		for (b2 = i - 2; b2 >= b1; b2 -= 2) {
-			switch (equation[b2].token.operatr) {
-			case PLUS:
-			case MINUS:
-				goto has_plus;
-			}
+		if (ii != 8) {
+			if ((i - b1) > 1 && equation[i+1].token.constant > 2.0
+			    && fmod(equation[i+1].token.constant, 1.0) != 0.0)
+				break;
+			if ((i - b1) == 1 && equation[b1].kind != CONSTANT)
+				return false;
 		}
-		return false;
-
-has_plus:
 		d1 = ceil(equation[i+1].token.constant) - 1.0;
-		if (d1 < 1.0) {
-			printf(_("Error in ceil() function!\n"));
-			longjmp(jmp_save, 13);
-		}
 		d2 = d1 * ((i - b1) + 1.0);
-		if (*np + d2 > n_tokens) {
-			error_huge();
+		if ((*np + d2) > (n_tokens - 10)) {
+			break;	/* too big to expand, do nothing */
 		}
 		j = d1;
 		k = d2;
@@ -633,6 +623,7 @@ int		*np, i;
 	return true;
 }
 
+#if	true
 /*
  * Convert a/(x^y) to a*(1/x)^y.
  * Return true if equation side is modified.
@@ -654,7 +645,7 @@ int		*np;
 	return modified;
 }
 
-int
+static int
 usp2_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
@@ -663,6 +654,9 @@ int		*np, i;
 	int	j, k;
 
 	level = equation[i].level;
+	if (equation[i+1].level == level && equation[i+1].kind == CONSTANT) {
+		return false;
+	}
 	for (j = i - 2;; j -= 2) {
 		if (j < 0)
 			return false;
@@ -693,6 +687,7 @@ int		*np, i;
 	equation[j].token.operatr = DIVIDE;
 	return true;
 }
+#endif
 
 /*
  * Convert anything times a negative constant to a positive constant
@@ -734,190 +729,39 @@ int		*np;
 }
 
 /*
- * Return the Greatest Common Divisor of doubles "d1" and "d2".
- * Will work with non-integers, but there may be some floating point error.
- * Always works correctly with integers.
- * Returns 0 if it failed.
+ * Convert 1/(k1^k2) to 1/(k1^(k2-1))/k1 if k1 is integer,
+ * otherwise 1*((1/k1)^k2).
  */
-double
-gcd(d1, d2)
-double	d1, d2;
-{
-	int	count;
-	double	larger;
-	double	divisor;
-	double	r1;
-	double	d;
-
-	d1 = fabs(d1);
-	d2 = fabs(d2);
-	if (d1 > d2) {
-		larger = d1;
-		divisor = d2;
-	} else {
-		larger = d2;
-		divisor = d1;
-	}
-	d = larger * epsilon;
-	if ((2 * d) >= divisor)
-		return 0.0;
-	for (count = 1; count < 50; count++) {
-		r1 = fmod(larger, divisor);
-		if (r1 <= d)
-			return divisor;
-		if ((divisor - r1) <= d)
-			return divisor;
-		larger = divisor;
-		divisor = r1;
-	}
-	return 0.0;
-}
-
-/*
- * Round a double to the nearest integer.
- */
-double
-my_round(d1)
-double	d1;
-{
-	if (d1 >= 0.0) {
-		modf(d1 + 0.5, &d1);
-	} else {
-		modf(d1 - 0.5, &d1);
-	}
-	return d1;
-}
-
-/*
- * Convert the passed double "d" to a fully reduced fraction.
- * "d1p" points to the numerator and "d2p" points to the denominator.
- * Return true if conversion was successful.
- */
-int
-f_to_fraction(d, d1p, d2p)
-double	d;
-double	*d1p;
-double	*d2p;
-{
-	double	divisor;
-	double	d1, d2;
-	k_type	k3, k4;
-
-	*d1p = d;
-	*d2p = 1.0;
-	if (fmod(d, 1.0) == 0.0)
-		return false;
-	if ((divisor = gcd(1.0, d)) > epsilon) {
-		d1 = d / divisor;
-		d2 = 1.0 / divisor;
-		d1 = my_round(d1);
-		d2 = my_round(d2);
-		if (fabs(d1) >= 1.0e10)
-			return false;
-		if (d2 >= 1.0e10)
-			return false;
-		if (d2 < 1.5)
-			return false;
-		divisor = gcd(d1, d2);
-		if (fmod(divisor, 1.0) != 0.0) {
-			printf(_("Error in gcd() function!\n"));
-			return false;
-		}
-		if (divisor > 1.0) {
-			d1 = d1 / divisor;
-			d2 = d2 / divisor;
-		}
-		k3 = (d1 / d2);
-		k4 = d;
-#if	EPSILON
-		if (fabs(k3 - k4) > (small_epsilon * fabs(k3))) {
-			return false;
-		}
-#else
-		if (k3 != k4)
-			return false;
-#endif
-		*d1p = d1;
-		*d2p = d2;
-		return true;
-	}
-	return false;
-}
-
-/*
- * Convert non-integer constants in an expression
- * to fractions when appropriate.
- */
-make_fractions(equation, np)
+patch_root_div(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	i, j, k;
+	int	i;
 	int	level;
-	double	d1, d2;
-	int	inc_level;
 
-	for (i = 0; i < *np; i += 2) {
-		if (equation[i].kind == CONSTANT) {
-			level = equation[i].level;
-			if (i > 0 && equation[i-1].level == level
-			    && equation[i-1].token.operatr == DIVIDE)
-				continue;
-			if (!f_to_fraction((double) equation[i].token.constant, &d1, &d2))
-				continue;
-			if ((*np + 2) > n_tokens) {
-				error_huge();
-			}
-			inc_level = (*np > 1);
-			if ((i + 1) < *np && equation[i+1].level == level) {
-				switch (equation[i+1].token.operatr) {
-				case TIMES:
-					if (d1 == 1.0) {
-						blt(&equation[i], &equation[i+2], (*np - (i + 2)) * sizeof(*equation));
-						*np -= 2;
-					} else {
-						equation[i].token.constant = d1;
-					}
-					for (j = i + 1; j < *np && equation[j].level >= level; j += 2) {
-						if (equation[j].level == level && equation[j].token.operatr == DIVIDE) {
-							break;
-						}
-					}
-					blt(&equation[j+2], &equation[j], (*np - j) * sizeof(*equation));
+	for (i = 1; i < *np - 2; i += 2) {
+		if (equation[i].token.operatr == DIVIDE) {
+			level = equation[i].level + 1;
+			if (equation[i+2].token.operatr == POWER
+			    && equation[i+2].level == level
+			    && equation[i+1].kind == CONSTANT
+			    && equation[i+3].level == level
+			    && equation[i+3].kind == CONSTANT) {
+				if (fmod(equation[i+1].token.constant, 1.0) == 0.0) {
+					if (*np + 2 > n_tokens)
+						error_huge();
+					equation[i+3].token.constant -= 1.0;
+					blt(&equation[i+2], &equation[i], (*np - i) * sizeof(token_type));
 					*np += 2;
-					equation[j].level = level;
-					equation[j].kind = OPERATOR;
-					equation[j].token.operatr = DIVIDE;
-					j++;
-					equation[j].level = level;
-					equation[j].kind = CONSTANT;
-					equation[j].token.constant = d2;
-					if (d1 == 1.0) {
-						i -= 2;
-					}
-					continue;
-				case DIVIDE:
-					inc_level = false;
-				default:
-					break;
+					equation[i+1].level = level - 1;
+					equation[i+1].kind = CONSTANT;
+					equation[i+1].token.constant = equation[i+3].token.constant;
+					i += 2;
+				} else {
+					equation[i].token.operatr = TIMES;
+					equation[i+1].token.constant = 1.0 / equation[i+1].token.constant;
 				}
 			}
-			j = i;
-			blt(&equation[i+3], &equation[i+1], (*np - (i + 1)) * sizeof(*equation));
-			*np += 2;
-			equation[j].token.constant = d1;
-			j++;
-			equation[j].level = level;
-			equation[j].kind = OPERATOR;
-			equation[j].token.operatr = DIVIDE;
-			j++;
-			equation[j].level = level;
-			equation[j].kind = CONSTANT;
-			equation[j].token.constant = d2;
-			if (inc_level) {
-				for (k = i; k <= j; k++)
-					equation[k].level++;
-			}
 		}
 	}
 }
